/* 
 * Copyright (C) 2009 HMI Group, University of Twente.
 *
 * @author Franois L.A. Knoppel <[alias] francois [on server at] knoppel.nl>
 * @author Almer S. Tigelaar <[alias] almer [on server at] gnome.org>
 * @author Danny Oude Bos <[alias] d.oudebos [on server at] utwente.nl>
 * @author Thijs Alofs <[alias] t.alofs [on server at] alumnus.utwente.nl>
 *
 * This file is part of the DEIRA project, which is free software; you can redistribute it
 * and/or modify it under the terms of the GNU "Lesser" General Public License v3 as published by the 
 * Free Software Foundation.
 */

package nl.utwente.ewi.hmi.deira.db;

import nl.utwente.ewi.hmi.deira.mmm.EmotionalCharacteristics;
import nl.utwente.ewi.hmi.deira.tgm.Grammar;

/**
 * Draft version of the Personality DataBase. The PersonalityDB represents the
 * EmotionalCharacteristics, Face and Grammar of the reporter.
 */
public class PersonalityDB {
	private EmotionalCharacteristics ec;
	/** < The EmotionalCharacteristics of the personality/reporter. */
	private Grammar grammar;
	/** < The Grammar of the personality/reporter. */
	private String face;
	/** < The String name representing the face of the personality/reporter. */
	private String voice;

	/** < The name of the voice to use. */

	/**
	 * Constructor of PersonalityDB.
	 * 
	 * @param e
	 *            The EmotionalCharacteristics to set.
	 * @param g
	 *            The grammar to set.
	 * @param voice
	 *            The voice to set.
	 * @param face
	 *            The face to set.
	 */
	public PersonalityDB(EmotionalCharacteristics e, Grammar g, String face,
			String voice) {
		this.ec = e;
		this.grammar = g;
		this.setFace(face);
		this.setVoice(voice);
	}

	// ------public methods--------
	/**
	 * @return Returns the EmotionalCharacteristics.
	 */
	public EmotionalCharacteristics getEmotionalCharacteristics() {
		return ec;
	}

	/**
	 * @return Returns the Grammar.
	 */
	public Grammar getGrammar() {
		return grammar;
	}

	/**
	 * @return Returns the String representation of the voice.
	 */
	public String getVoice() {
		return voice;
	}

	/**
	 * @return Returns the String representation of the face.
	 */
	public String getFace() {
		return face;
	}

	public void setVoice(String voice) {
		this.voice = voice;
	}

	public void setFace(String face) {
		this.face = face;
	}

}