/* 
 * Copyright (C) 2009 HMI Group, University of Twente.
 *
 * @author Franois L.A. Knoppel <[alias] francois [on server at] knoppel.nl>
 * @author Almer S. Tigelaar <[alias] almer [on server at] gnome.org>
 * @author Danny Oude Bos <[alias] d.oudebos [on server at] utwente.nl>
 * @author Thijs Alofs <[alias] t.alofs [on server at] alumnus.utwente.nl>
 *
 * This file is part of the DEIRA project, which is free software; you can redistribute it
 * and/or modify it under the terms of the GNU "Lesser" General Public License v3 as published by the 
 * Free Software Foundation.
 */

package nl.utwente.ewi.hmi.deira.fam;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map.Entry;

import nl.utwente.ewi.hmi.deira.generic.Module;
import nl.utwente.ewi.hmi.deira.mmm.EmotionalState;
import nl.utwente.ewi.hmi.deira.mmm.MMM;
import nl.utwente.ewi.hmi.deira.om.Outputter;
import nl.utwente.ewi.hmi.deira.queue.EventQueue;

public abstract class FacialAnimator extends Thread implements Module {

	protected MMM mmm;
	/** < Internal reference to the Mental Model Module. */
	protected EventQueue queue;
	/** < Internal reference to the event queue. */
	protected boolean terminate;
	/** < Used to indicate that the thread should stop running. */
	protected Outputter om;
	protected String moduleName;

	/** < Internal reference to the output module */

	public FacialAnimator(String string) {
		super(string);
	}

	/**
	 * Sets the outputter to use for direct output of facial animations.
	 * 
	 * @param om
	 */
	public void setOutputter(Outputter om) {
		this.om = om;
	}

	public abstract void close();

	public String getModuleName() {
		return moduleName;
	}

	protected String addDominantEmotions(ArrayList<String> expressions) {
		EmotionalState es = mmm.getEmotionalState();
		HashMap<String, Double> emotionalLevels = es.getEmotionalLevels();

		double tensionLevel = es.getEmotionalLevel("tension");
		if (tensionLevel < 0.3) {
			expressions.add("tension_low");
		} else if (tensionLevel < 0.6) {
			expressions.add("tension_medium");
		} else {
			expressions.add("tension_high");
		}
		emotionalLevels.remove("tension");

		String dominantEmotion = "";
		Double dominantValue = -1.0;

		for (Entry<String, Double> entry : emotionalLevels.entrySet()) {
			Double value = entry.getValue();
			if (value > dominantValue) {
				dominantValue = value;
				dominantEmotion = entry.getKey();
			}
		}

		String dominantExpression = "";
		if (dominantEmotion.equals("surprise")) {
			dominantExpression = "surprise";
			if (dominantValue > 0.6) {
				dominantExpression += "_intense";
			}
			if (Math.random() > 0.5) {
				dominantExpression += "_short";
			}
		} else if (dominantEmotion.equals("amusement")) {
			dominantExpression = "smile";
			if (dominantValue > 0.6) {
				dominantExpression += "_open";
			} else {
				dominantExpression += "_closed";
			}

			double random = Math.random();

			if (random < 0.3) {
				dominantExpression += "_1s";
			} else if (random < 0.7) {
				dominantExpression += "_3s";
			} else {
				dominantExpression = "happy";
			}
		} else if (dominantEmotion.equals("anger")) {
			dominantExpression = "anger";
			if (dominantValue > 0.6) {
				dominantExpression += "_intense";
			}
		} else if (dominantEmotion.equals("sadness")) {
			dominantExpression = "sadness";
			if (dominantValue > 0.6) {
				dominantExpression += "_intense";
			}
		} else if (dominantEmotion.equals("pity")) {
			dominantExpression = "pity";
			if (dominantValue > 0.6) {
				dominantExpression += "_intense";
			}
		}
		expressions.add(dominantExpression);
		return dominantExpression;
	}

}