/* 
 * Copyright (C) 2009 HMI Group, University of Twente.
 *
 * @author Franois L.A. Knoppel <[alias] francois [on server at] knoppel.nl>
 * @author Almer S. Tigelaar <[alias] almer [on server at] gnome.org>
 * @author Danny Oude Bos <[alias] d.oudebos [on server at] utwente.nl>
 * @author Thijs Alofs <[alias] t.alofs [on server at] alumnus.utwente.nl>
 *
 * This file is part of the DEIRA project, which is free software; you can redistribute it
 * and/or modify it under the terms of the GNU "Lesser" General Public License v3 as published by the 
 * Free Software Foundation.
 */

package nl.utwente.ewi.hmi.deira.fam;

import java.util.ArrayList;
import java.util.TimerTask;
import java.util.logging.Logger;

import nl.utwente.ewi.hmi.deira.mmm.EmotionalState;
import nl.utwente.ewi.hmi.deira.mmm.MMM;
import nl.utwente.ewi.hmi.deira.om.OutputEvent;
import nl.utwente.ewi.hmi.deira.om.Outputter;

public class TimedHeadMotionEvent extends TimerTask {
	private MMM mmm;
	/** < Internal reference to the Mental Model Module. */
	private Outputter om;
	/** < Internal reference to the Output Module */
	private long interval;
	/** < Invocation interval */
	private long timeStamp;
	/** < Current timeStamp */
	private long lastAnimationTime;
	/** < Moment in time the last headmotion animation was sent */
	private static Logger log = Logger.getLogger("deira.fam");
	/** < Logging instance */

	private final long DURATION_HEAD_MOTION = 9000; /*
													 * Reduction causes slight
													 * overlap between motion
													 * animations
													 */

	TimedHeadMotionEvent(Outputter om, MMM mmm) {
		log.info("FAM/FTE: Created");

		this.om = om;
		this.mmm = mmm;
		this.timeStamp = 0;
		this.lastAnimationTime = -(DURATION_HEAD_MOTION * 2);
	}

	public void setRunInterval(long interval) {
		this.interval = interval;
	}

	public void run() {
		EmotionalState es = mmm.getEmotionalState();
		double tension = es.getAveragedEmotionalLevel("tension");

		if (this.timeStamp - this.lastAnimationTime > DURATION_HEAD_MOTION) { /*
																			 * Head
																			 * motions
																			 * last
																			 * ten
																			 * seconds
																			 */
			String HMFile = "";

			if (tension <= 0.25) {
				HMFile = "headmotion_verylow";
			} else if (tension > 0.25 && tension <= 0.50) {
				HMFile = "headmotion_low";
			} else if (tension > 0.50 && tension <= 0.75) {
				HMFile = "headmotion_medium";
			} else if (tension > 0.75) {
				HMFile = "headmotion_high";
			}

			ArrayList<String> expressions = new ArrayList<String>();
			expressions.add(HMFile);

			log.info("FAM/FTE: Sending Headmotion Animation");
			om.sendDirectAnimations(expressions);
			OutputEvent outputEvent = om.getOutputEvent();
			if (outputEvent != null) {
				outputEvent.OnOutputFacialTimeEvent(tension, HMFile);
			}

			lastAnimationTime = timeStamp;
		}

		this.timeStamp += this.interval; /* Increase millisecond timestamp */
	}
}