/* 
 * Copyright (C) 2009 HMI Group, University of Twente.
 *
 * @author Franois L.A. Knoppel <[alias] francois [on server at] knoppel.nl>
 * @author Almer S. Tigelaar <[alias] almer [on server at] gnome.org>
 * @author Danny Oude Bos <[alias] d.oudebos [on server at] utwente.nl>
 * @author Thijs Alofs <[alias] t.alofs [on server at] alumnus.utwente.nl>
 *
 * This file is part of the DEIRA project, which is free software; you can redistribute it
 * and/or modify it under the terms of the GNU "Lesser" General Public License v3 as published by the 
 * Free Software Foundation.
 */

package nl.utwente.ewi.hmi.deira.fam;

import java.util.ArrayList;
import java.util.Random;
import java.util.logging.Logger;

import nl.utwente.ewi.hmi.deira.generic.DeiraThread;
import nl.utwente.ewi.hmi.deira.om.OutputEvent;

public class VSAudienceLookingAnimator extends DeiraThread {
	private static Logger log = Logger.getLogger("deira.fam");
	private VSFacialAnimator facialAnimator;
	private boolean work = true;
	private static final int MINSLEEPTIME = 400;
	private static final int MAXSLEEPTIME = 20000;
	private static final int LOOKMEAN = 1000;
	private static final int LOOKDEV = 800;
	private double lookingDirectionAmplitude = 5.0;
	private double mutationFactor = 4.0;
	private Random randomGenerator = new Random();

	public VSAudienceLookingAnimator(String name,
			VSFacialAnimator facialAnimator) {
		super(name);
		this.facialAnimator = facialAnimator;
	}

	public void run() {
		int sleepTime = 100;

		double previousLookingDirection = 0.0;
		while (work) {

			try {
				synchronized (this) {
					this.wait(sleepTime);

					ArrayList<String> expressions = new ArrayList<String>();

					/* Crowd looking */
					double newLookingDirection = getNewLookingDirection(
							previousLookingDirection, sleepTime);
					int discreteLookingDirection = Math
							.round((float) newLookingDirection);
					int discretePreviousLookingDirection = Math
							.round((float) previousLookingDirection);
					previousLookingDirection = newLookingDirection;
					if (discreteLookingDirection != discretePreviousLookingDirection) {

						String lookAnimation = "";
						if (discreteLookingDirection == 0) {
							lookAnimation = "look_center";
						} else if (discreteLookingDirection > 0) {
							lookAnimation = "look_right"
									+ discreteLookingDirection;
						} else if (discreteLookingDirection < 0) {
							lookAnimation = "look_left"
									+ -discreteLookingDirection;
						}

						expressions.add(lookAnimation);

						log.info("VSFAM/FTE: Sending Headmotion Animation: "
								+ lookAnimation);
						facialAnimator.om.sendDirectAnimations(expressions);
						OutputEvent outputEvent = facialAnimator.om
								.getOutputEvent();
						if (outputEvent != null) {
							outputEvent.OnOutputFacialTimeEvent(1.0,
									lookAnimation);
						}
					}

					sleepTime = this.determineSleepTime();
				}
			} catch (InterruptedException e) {

				log.severe(e.toString());
			}
		}

	}

	private double getNewLookingDirection(double previousLookingDirection,
			double timePassed) {

		double timeFactor = Math.max(1, Math.min(timePassed / 400, 4));
		double lookDelta = ((Math.random() * 2) - 1)
				* lookingDirectionAmplitude * mutationFactor / timeFactor;

		double newLookingDirection = previousLookingDirection + lookDelta;

		if (newLookingDirection > lookingDirectionAmplitude) {
			newLookingDirection = Math.max(-lookingDirectionAmplitude,
					(2 * lookingDirectionAmplitude) - newLookingDirection);
		} else if (newLookingDirection < -lookingDirectionAmplitude) {
			newLookingDirection = Math.min(lookingDirectionAmplitude,
					(2 * -lookingDirectionAmplitude) - newLookingDirection);
		}

		return newLookingDirection;
	}

	private int determineSleepTime() {

		int sleepTime = (int) (LOOKMEAN + (this.randomGenerator.nextGaussian() * LOOKDEV));

		if (sleepTime > MAXSLEEPTIME) {
			sleepTime = MAXSLEEPTIME;
		} else if (sleepTime < MINSLEEPTIME) {
			sleepTime = MINSLEEPTIME;
		}

		return sleepTime;
	}

	@Override
	public void close() {
		work = false;
	}
}