/* 
 * Copyright (C) 2009 HMI Group, University of Twente.
 *
 * @author Franois L.A. Knoppel <[alias] francois [on server at] knoppel.nl>
 * @author Almer S. Tigelaar <[alias] almer [on server at] gnome.org>
 * @author Danny Oude Bos <[alias] d.oudebos [on server at] utwente.nl>
 * @author Thijs Alofs <[alias] t.alofs [on server at] alumnus.utwente.nl>
 *
 * This file is part of the DEIRA project, which is free software; you can redistribute it
 * and/or modify it under the terms of the GNU "Lesser" General Public License v3 as published by the 
 * Free Software Foundation.
 */

package nl.utwente.ewi.hmi.deira.fam;

import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.logging.Logger;

import nl.utwente.ewi.hmi.deira.mmm.MMM;
import nl.utwente.ewi.hmi.deira.om.HaptekOutputter;
import nl.utwente.ewi.hmi.deira.queue.Event;
import nl.utwente.ewi.hmi.deira.queue.EventQueue;

public class VSFacialAnimator extends FacialAnimator {
	private static Logger log = Logger.getLogger("deira.fam");
	private VSAudienceLookingAnimator crowdAnimator;
	private VSGlancesAnimator glanceAnimator;

	/** < Logging instance */

	/**
	 * Constructs a new FacialAnimator.
	 * 
	 * @param eq
	 *            reference to the eventqueue.
	 * @param mmm
	 *            reference to the mental model module.
	 */
	public VSFacialAnimator(EventQueue eq, MMM mmm) {
		super("DEIRA VSFAM");

		this.queue = eq;
		this.mmm = mmm;
		this.om = null;

		moduleName = "VSFAM";
	}

	/**
	 * Sets the outputter to use for direct output of facial animations.
	 * 
	 * @param om
	 */
	public void setOutputter(HaptekOutputter om) {
		this.om = om;
	}

	/**
	 * Runs the FacialAnimator thread. This will wait for events and attach
	 * facial expression information to the events.
	 */
	public void run() {
		this.terminate = false;

		this.crowdAnimator = new VSAudienceLookingAnimator("DEIRA VSFAM.crowd",
				this);
		this.crowdAnimator.start();

		// this.glanceAnimator = new VSGlancesAnimator("DEIRA VSFAM.glances",
		// this);
		// this.glanceAnimator.start();

		do {
			try {
				this.queue.waitForEvent();
			} catch (Exception e) {
				log.severe("VSFAM: Error waiting for event! " + e);
			}

			Event event = this.queue.peek();

			if (event != null) {
				log.info("VSFAM: Event grabbed (" + event.getId() + ")");
				ArrayList<String> expressions = new ArrayList<String>();

				// =- Do normal facial animation
				/* Find out the dominant emotion */
				String dominantExpression = addDominantEmotions(expressions);

				event.tag("expressions", expressions);
				log
						.info(MessageFormat
								.format(
										"VSFAM: Event processed ({0}) -- Dominant expression: {1}",
										event.getId(), dominantExpression));
				this.queue.update(event);

			} else {
				log.severe("VSFAM: Woken while nothing was available!");
			}
		} while (!this.terminate);
	}

	/**
	 * Instruct the FacialAnimator thread to stop waiting for events and
	 * terminate.
	 */
	public void close() {
		if (this.crowdAnimator != null) {
			this.crowdAnimator.close();
		}
		if (this.glanceAnimator != null) {
			this.glanceAnimator.close();
		}
		this.terminate = true;
	}

}
