/* 
 * Copyright (C) 2009 HMI Group, University of Twente.
 *
 * @author Franois L.A. Knoppel <[alias] francois [on server at] knoppel.nl>
 * @author Almer S. Tigelaar <[alias] almer [on server at] gnome.org>
 * @author Danny Oude Bos <[alias] d.oudebos [on server at] utwente.nl>
 * @author Thijs Alofs <[alias] t.alofs [on server at] alumnus.utwente.nl>
 *
 * This file is part of the DEIRA project, which is free software; you can redistribute it
 * and/or modify it under the terms of the GNU "Lesser" General Public License v3 as published by the 
 * Free Software Foundation.
 */

package nl.utwente.ewi.hmi.deira.fam;

import java.util.ArrayList;
import java.util.Random;
import java.util.logging.Logger;

import nl.utwente.ewi.hmi.deira.generic.DeiraThread;
import nl.utwente.ewi.hmi.deira.om.OutputEvent;

public class VSGlancesAnimator extends DeiraThread {
	private static Logger log = Logger.getLogger("deira.fam");
	private VSFacialAnimator facialAnimator;
	private boolean work = true;

	private static final int MINSLEEPTIME = 3500;
	private static final int MAXSLEEPTIME = 6500;
	private static final int SLEEPMEAN = 5000;
	private static final int SLEEPDEV = 1000;

	private static final int MINGLANCETIME = 200;
	private static final int MAXGLANCETIME = 6000;
	private static final int GLANCEMEAN = 1100;
	private static final int GLANCEDEV = 1000;

	private Random randomGenerator = new Random();

	public VSGlancesAnimator(String name, VSFacialAnimator facialAnimator) {
		super(name);
		this.facialAnimator = facialAnimator;
	}

	public void run() {
		int sleepTime = 100;

		String[] glances = { "glance_up_right", "glance_up_middle",
				"glance_up_left", "glance_down_right", "glance_down_middle",
				"glance_down_left" };

		while (work) {

			try {
				synchronized (this) {
					this.wait(sleepTime);

					ArrayList<String> expressions = new ArrayList<String>();

					/* Glances */
					int rand = (int) (randomGenerator.nextFloat() * glances.length);

					String glanceAnimation = glances[rand];
					expressions.add(glanceAnimation);
					facialAnimator.om.sendDirectAnimations(expressions);

					// =-- FIXME hack
					int variableDuration = determineDuration();
					this.wait(variableDuration);

					expressions = new ArrayList<String>();
					String glanceStop = "glance_stop2";
					expressions.add(glanceStop);

					// HashMap<String, Double> parameters = new HashMap<String,
					// Double>();
					// parameters.put("duration",
					// 0.1);//0.5+variableDuration/1000 );

					log.info("VSFAM/FTE: Sending Headmotion Animation: "
							+ glanceAnimation);
					facialAnimator.om.sendDirectAnimations(expressions);
					OutputEvent outputEvent = facialAnimator.om
							.getOutputEvent();
					if (outputEvent != null) {
						outputEvent.OnOutputFacialTimeEvent(1.0,
								glanceAnimation);
					}

					sleepTime = this.determineSleepTime();
				}
			} catch (InterruptedException e) {

				log.severe(e.toString());
			}
		}

	}

	private int determineSleepTime() {

		int sleepTime = (int) (SLEEPMEAN + (this.randomGenerator.nextGaussian() * SLEEPDEV));

		if (sleepTime > MAXSLEEPTIME) {
			sleepTime = MAXSLEEPTIME;
		} else if (sleepTime < MINSLEEPTIME) {
			sleepTime = MINSLEEPTIME;
		}

		return sleepTime;
	}

	private int determineDuration() {

		int sleepTime = (int) (GLANCEMEAN + (this.randomGenerator
				.nextGaussian() * GLANCEDEV));

		if (sleepTime > MAXGLANCETIME) {
			sleepTime = MAXGLANCETIME;
		} else if (sleepTime < MINGLANCETIME) {
			sleepTime = MINGLANCETIME;
		}

		return sleepTime;
	}

	@Override
	public void close() {
		work = false;
	}
}