/* 
 * Copyright (C) 2009 HMI Group, University of Twente.
 *
 * @author Franois L.A. Knoppel <[alias] francois [on server at] knoppel.nl>
 * @author Almer S. Tigelaar <[alias] almer [on server at] gnome.org>
 * @author Danny Oude Bos <[alias] d.oudebos [on server at] utwente.nl>
 * @author Thijs Alofs <[alias] t.alofs [on server at] alumnus.utwente.nl>
 *
 * This file is part of the DEIRA project, which is free software; you can redistribute it
 * and/or modify it under the terms of the GNU "Lesser" General Public License v3 as published by the 
 * Free Software Foundation.
 */

package nl.utwente.ewi.hmi.deira.gui;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map.Entry;

import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.events.PaintListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.Canvas;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;

/**
 * The emotion graph panel displays the values of emotions as the evolve over
 * time. It exists in addition to the sliders that display the emotion at
 * present.
 */
class EmotionGraphCanvas extends Canvas implements PaintListener {
	private HashMap<String, ArrayList<Double>> emotions = new HashMap<String, ArrayList<Double>>();
	private int graphWidth;
	private int graphHeight;
	private int segmentcount = 100;
	private int segmentLength;
	private DEIRASWT gui;

	public EmotionGraphCanvas(Composite composite, int i, DEIRASWT gui) {
		super(composite, i);
		this.gui = gui;
		addPaintListener(this);
	}

	public synchronized void paintArrayList(GC gc, ArrayList<Double> values,
			Color color) {
		gc.setAntialias(1);
		gc.setForeground(color);

		int lastY = 0;
		int lastX = 0;
		int x = -segmentLength;
		int y = 0;

		for (double value : values) {
			lastX = x;
			lastY = y;

			x += segmentLength;
			y = graphHeight - (int) (graphHeight * value);

			if (lastX >= 0) {
				gc.drawLine(lastX, lastY, x, y);
			}
		}
	}

	public void paintControl(PaintEvent pe) {
		GC gc = pe.gc;

		Point size = getSize();
		graphWidth = size.x - 1;
		graphHeight = size.y - 1;
		segmentLength = graphWidth / segmentcount;

		gc.setBackground(new Color(Display.getDefault(), 255, 255, 255));
		gc.setForeground(new Color(Display.getDefault(), 188, 188, 188));
		gc.fillRectangle(0, 0, graphWidth, graphHeight);
		gc.drawRectangle(0, 0, graphWidth, graphHeight);

		for (Entry<String, ArrayList<Double>> emotion : emotions.entrySet()) {
			ArrayList<Double> value = emotion.getValue();
			String key = emotion.getKey();
			Color color = gui.emoColors.get(key);
			paintArrayList(gc, value, color);
		}
	}

	public synchronized void addEmotions(HashMap<String, Double> newEmotions) {
		if (newEmotions != null) {
			for (Entry<String, Double> emotion : newEmotions.entrySet()) {
				String key = emotion.getKey();
				Double value = emotion.getValue();
				ArrayList<Double> currentlist = emotions.get(key);
				if (currentlist == null) {
					currentlist = new ArrayList<Double>();
					emotions.put(key, currentlist);
				}

				if (currentlist.size() > segmentcount) {
					currentlist.remove(0);
				}

				currentlist.add(value);

			}
		}
	}
}