/* 
 * Copyright (C) 2009 HMI Group, University of Twente.
 *
 * @author Franois L.A. Knoppel <[alias] francois [on server at] knoppel.nl>
 * @author Almer S. Tigelaar <[alias] almer [on server at] gnome.org>
 * @author Danny Oude Bos <[alias] d.oudebos [on server at] utwente.nl>
 * @author Thijs Alofs <[alias] t.alofs [on server at] alumnus.utwente.nl>
 *
 * This file is part of the DEIRA project, which is free software; you can redistribute it
 * and/or modify it under the terms of the GNU "Lesser" General Public License v3 as published by the 
 * Free Software Foundation.
 */

package nl.utwente.ewi.hmi.deira.gui;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.util.ArrayList;

import javax.swing.JPanel;

/**
 * The emotion graph panel displays the values of emotions as the evolve over
 * time. It exists in addition to the sliders that display the emotion at
 * present.
 */
class EmotionGraphPanel extends JPanel {
	private ArrayList<Integer> tension;
	private ArrayList<Integer> amusement;
	private ArrayList<Integer> surprise;
	private ArrayList<Integer> pity;

	private final int graphWidth = 250;
	private final int graphHeight = 100;
	private final int segmentLength = 5;

	public EmotionGraphPanel() {
		setPreferredSize(new Dimension(graphWidth, graphHeight));
		this.tension = new ArrayList<Integer>();
		this.amusement = new ArrayList<Integer>();
		this.surprise = new ArrayList<Integer>();
		this.pity = new ArrayList<Integer>();
	}

	public synchronized void paintArrayList(Graphics g,
			ArrayList<Integer> values) {
		int lastY = 0;
		int lastX = 0;
		int x = -segmentLength;
		int y = 0;

		for (int value : values) {
			lastX = x;
			lastY = y;

			x += segmentLength;
			y = graphHeight
					- new Double((graphHeight / 100.0) * value).intValue();

			if (lastX >= 0) {
				g.drawLine(lastX, lastY, x, y);
			}
		}
	}

	public void paint(Graphics g) {
		/* Paint white background */
		g.setColor(Color.white);
		g.fillRect(0, 0, graphWidth - 1, graphHeight - 1);

		/* Paint all emotion vectors */
		g.setColor(Color.blue);
		this.paintArrayList(g, this.tension);

		g.setColor(Color.green);
		this.paintArrayList(g, this.surprise);

		g.setColor(Color.magenta);
		this.paintArrayList(g, this.amusement);

		g.setColor(Color.red);
		this.paintArrayList(g, this.pity);

		/* Paint outer border rectangle */
		g.setColor(Color.black);
		g.drawRect(0, 0, graphWidth - 1, graphHeight - 1);
	}

	public synchronized void addEmotions(int tension, int surprise,
			int amusement, int pity) {
		this.tension.add(tension);
		this.surprise.add(surprise);
		this.amusement.add(amusement);
		this.pity.add(pity);

		if (this.tension.size() > (graphWidth / segmentLength) + 1) {
			this.tension.remove(0);
			this.surprise.remove(0);
			this.amusement.remove(0);
			this.pity.remove(0);
		}
		this.repaint();
	}
}