/* 
 * Copyright (C) 2009 HMI Group, University of Twente.
 *
 * @author Franois L.A. Knoppel <[alias] francois [on server at] knoppel.nl>
 * @author Almer S. Tigelaar <[alias] almer [on server at] gnome.org>
 * @author Danny Oude Bos <[alias] d.oudebos [on server at] utwente.nl>
 * @author Thijs Alofs <[alias] t.alofs [on server at] alumnus.utwente.nl>
 *
 * This file is part of the DEIRA project, which is free software; you can redistribute it
 * and/or modify it under the terms of the GNU "Lesser" General Public License v3 as published by the 
 * Free Software Foundation.
 */

package nl.utwente.ewi.hmi.deira.iam.riam;

import java.util.ArrayList;

/**
 * This class represents the actors as described in the event generation rule.
 * When interpreted it returns a ArrayList of Participants.
 * 
 * Part of EventGenerationEvent.
 */
public class EventGenerationEventActors {
	private ArrayList<Integer> actors = null;

	/**
	 * < A ArrayList with Integers indicating the Participants for this event by
	 * their ranks.
	 */

	/**
	 * Constructor.
	 * 
	 * @param actors
	 *            A ArrayList with Integers indicating the Participants for this
	 *            event by their current ranks.
	 */
	public EventGenerationEventActors(ArrayList<Integer> actors) {
		this.actors = actors;
		if (this.actors == null)
			this.actors = new ArrayList<Integer>();
	}

	/**
	 * Interprets the actor part of the generation rule, resulting in a
	 * ArrayList with actors.
	 * 
	 * @param raceState
	 *            The (current) raceState for which the participating actors
	 *            need to be derived.
	 * @return ArrayList with participating actors.
	 */
	public ArrayList<Participant> interpret(ArrayList<Participant> raceState) {
		ArrayList<Participant> a = new ArrayList<Participant>();
		Participant[] participant = new Participant[4];

		for (int i = 0; i < raceState.size(); i++) {
			State s = raceState.get(i).getCurrentState();
			if (s != null)
				participant[s.getRank() - 1] = raceState.get(i);
			else {
				participant[0] = raceState.get(0);
				participant[1] = raceState.get(1);
				participant[2] = raceState.get(2);
				participant[3] = raceState.get(3);
			}
		}

		for (int i = 0; i < actors.size(); i++) {
			a.add(participant[actors.get(i) - 1]);
		}
		return a;
	}

	/**
	 * Function to create a String representation of the participating actors by
	 * rank number. Mainly for debugging purposes.
	 * 
	 * @return A String representation showing the participating actors by rank
	 *         number.
	 */
	public String toString() {
		String s = "< ";
		for (int i = 0; i < actors.size(); i++) {
			s += "#" + actors.get(i) + ", ";
		}
		if (s.length() > 2)
			s = s.substring(0, s.length() - 2);
		s += " >";
		return s;
	}

}
