/* 
 * Copyright (C) 2009 HMI Group, University of Twente.
 *
 * @author Franois L.A. Knoppel <[alias] francois [on server at] knoppel.nl>
 * @author Almer S. Tigelaar <[alias] almer [on server at] gnome.org>
 * @author Danny Oude Bos <[alias] d.oudebos [on server at] utwente.nl>
 * @author Thijs Alofs <[alias] t.alofs [on server at] alumnus.utwente.nl>
 *
 * This file is part of the DEIRA project, which is free software; you can redistribute it
 * and/or modify it under the terms of the GNU "Lesser" General Public License v3 as published by the 
 * Free Software Foundation.
 */

package nl.utwente.ewi.hmi.deira.iam.riam;

import java.util.ArrayList;

/**
 * This class represents the decay that belongs to an event as described in the
 * event generation rule. When interpreted it returns a float value indicating
 * the decay of importance per second.
 * 
 * Part of EventGenerationEvent.
 */
public class EventGenerationEventDecay {
	private EventGenerationExpression expr = null;

	/** < The expression that represents the decay. */

	/**
	 * Constructor.
	 * 
	 * @param expr
	 *            The expression that represents the decay.
	 */
	public EventGenerationEventDecay(EventGenerationExpression expr) {
		this.expr = expr;
	}

	/**
	 * Interprets the event decay of importance per second.
	 * 
	 * @param raceState
	 *            The (current) raceState for which the decay needs to be
	 *            interpreted.
	 * @return The event decay of importance per second.
	 */
	public float interpret(ArrayList<Participant> raceState) {
		return expr.interpret(raceState);
	}

	/**
	 * Function to create a String representation of this decay. Mainly for
	 * debugging purposes.
	 * 
	 * @return A String representation of this decay.
	 */
	public String toString() {
		return expr.toString();
	}

}
