/* 
 * Copyright (C) 2009 HMI Group, University of Twente.
 *
 * @author Franois L.A. Knoppel <[alias] francois [on server at] knoppel.nl>
 * @author Almer S. Tigelaar <[alias] almer [on server at] gnome.org>
 * @author Danny Oude Bos <[alias] d.oudebos [on server at] utwente.nl>
 * @author Thijs Alofs <[alias] t.alofs [on server at] alumnus.utwente.nl>
 *
 * This file is part of the DEIRA project, which is free software; you can redistribute it
 * and/or modify it under the terms of the GNU "Lesser" General Public License v3 as published by the 
 * Free Software Foundation.
 */

package nl.utwente.ewi.hmi.deira.iam.riam;

import java.util.ArrayList;

/**
 * This class represents the label that belongs to an event as described in the
 * event generation rule. When interpreted it returns the label, which is
 * effectively the subtype of this event.
 * 
 * Part of EventGenerationEvent.
 */
public class EventGenerationEventLabel {
	private String label = null;

	/** < The label indicating the subtype of this event. */

	/**
	 * Constructor.
	 * 
	 * @param label
	 *            The label indicating the subtype of this event.
	 */
	public EventGenerationEventLabel(String label) {
		this.label = label;
	}

	/**
	 * Interprets the event label.
	 * 
	 * @param raceState
	 *            The (current) raceState for which the label needs to be
	 *            interpreted.
	 * @return The event label.
	 */
	public String interpret(ArrayList<Participant> raceState) {
		return label;
	}

	/**
	 * Function to create a String representation of this label. Mainly for
	 * debugging purposes.
	 * 
	 * @return A String representation of this label.
	 */
	public String toString() {
		return label;
	}

}
