/* 
 * Copyright (C) 2009 HMI Group, University of Twente.
 *
 * @author Franois L.A. Knoppel <[alias] francois [on server at] knoppel.nl>
 * @author Almer S. Tigelaar <[alias] almer [on server at] gnome.org>
 * @author Danny Oude Bos <[alias] d.oudebos [on server at] utwente.nl>
 * @author Thijs Alofs <[alias] t.alofs [on server at] alumnus.utwente.nl>
 *
 * This file is part of the DEIRA project, which is free software; you can redistribute it
 * and/or modify it under the terms of the GNU "Lesser" General Public License v3 as published by the 
 * Free Software Foundation.
 */

package nl.utwente.ewi.hmi.deira.iam.riam;

import java.util.ArrayList;

/**
 * This class represents the optional parameters that belong to an event as
 * described in the event generation rule. When interpreted it returns a
 * ArrayList of Integers, one for each parameter specified in the rule.
 * 
 * Part of EventGenerationEvent.
 */
public class EventGenerationEventParams {
	private ArrayList<EventGenerationExpression> params = null;

	/**
	 * Constructor.
	 * 
	 * @param params
	 *            list of expressions indicating the passed on parameters.
	 */
	public EventGenerationEventParams(
			ArrayList<EventGenerationExpression> params) {
		this.params = params;
		if (this.params == null)
			this.params = new ArrayList<EventGenerationExpression>();
	}

	/**
	 * Interprets the event parameters.
	 * 
	 * @param raceState
	 *            The (current) raceState for which the parameters needs to be
	 *            interpreted.
	 * @return The event parameters.
	 */
	public ArrayList<Float> interpret(ArrayList<Participant> raceState) {
		ArrayList<Float> results = new ArrayList<Float>();
		for (EventGenerationExpression param : params) {
			results.add(param.interpret(raceState));
		}
		return results;
	}

	/**
	 * Function to create a String representation of these event parameters.
	 * Mainly for debugging purposes.
	 * 
	 * @return A String representation of these event parameters.
	 */
	public String toString() {
		String s = "< ";

		for (int i = 0; i < params.size(); i++) {
			s += params.get(i).toString() + ", ";
		}

		if (s.length() > 2)
			s = s.substring(0, s.length() - 2);

		s += " >";
		return s;
	}

}
