/* 
 * Copyright (C) 2009 HMI Group, University of Twente.
 *
 * @author Franois L.A. Knoppel <[alias] francois [on server at] knoppel.nl>
 * @author Almer S. Tigelaar <[alias] almer [on server at] gnome.org>
 * @author Danny Oude Bos <[alias] d.oudebos [on server at] utwente.nl>
 * @author Thijs Alofs <[alias] t.alofs [on server at] alumnus.utwente.nl>
 *
 * This file is part of the DEIRA project, which is free software; you can redistribute it
 * and/or modify it under the terms of the GNU "Lesser" General Public License v3 as published by the 
 * Free Software Foundation.
 */

package nl.utwente.ewi.hmi.deira.iam.riam;

import java.util.ArrayList;

/**
 * This class represents the type of an event as described in the event
 * generation rule. When interpreted it returns the event type. Note: the event
 * label indicates the subtype.
 * 
 * Part of EventGenerationEvent.
 */
public class EventGenerationEventType {
	private String type = null;

	/** < String indicating the event type. */

	/**
	 * Constructor.
	 * 
	 * @param type
	 *            String indicating the event type.
	 */
	public EventGenerationEventType(String type) {
		this.type = type;
	}

	/**
	 * Interprets the event type.
	 * 
	 * @param raceState
	 *            The (current) raceState for which the type needs to be
	 *            interpreted.
	 * @return The event type.
	 */
	public String interpret(ArrayList<Participant> raceState) {
		return type;
	}

	/**
	 * Function to create a String representation of this event type. Mainly for
	 * debugging purposes.
	 * 
	 * @return A String representation of this event type.
	 */
	public String toString() {
		return type;
	}

}
