/* 
 * Copyright (C) 2009 HMI Group, University of Twente.
 *
 * @author Franois L.A. Knoppel <[alias] francois [on server at] knoppel.nl>
 * @author Almer S. Tigelaar <[alias] almer [on server at] gnome.org>
 * @author Danny Oude Bos <[alias] d.oudebos [on server at] utwente.nl>
 * @author Thijs Alofs <[alias] t.alofs [on server at] alumnus.utwente.nl>
 *
 * This file is part of the DEIRA project, which is free software; you can redistribute it
 * and/or modify it under the terms of the GNU "Lesser" General Public License v3 as published by the 
 * Free Software Foundation.
 */

/* 
 * TODO also create subtypes for strings and booleans?
 */

package nl.utwente.ewi.hmi.deira.iam.riam;

import java.util.ArrayList;

/**
 * This class is a general interface for any expressions that may be described
 * in the event generation rules. When interpreted it returns a float value
 * corresponding to the expression.
 */
public interface EventGenerationExpression {
	/**
	 * Interprets the expression.
	 * 
	 * @param raceState
	 *            The (current) raceState for which the expression needs to be
	 *            interpreted.
	 * @return The expression value.
	 */
	public float interpret(ArrayList<Participant> raceState);

	/**
	 * Function to create a String representation of this expression. Mainly for
	 * debugging purposes.
	 * 
	 * @return A String representation of this expression.
	 */
	public String toString();
}
