/* 
 * Copyright (C) 2009 HMI Group, University of Twente.
 *
 * @author Franois L.A. Knoppel <[alias] francois [on server at] knoppel.nl>
 * @author Almer S. Tigelaar <[alias] almer [on server at] gnome.org>
 * @author Danny Oude Bos <[alias] d.oudebos [on server at] utwente.nl>
 * @author Thijs Alofs <[alias] t.alofs [on server at] alumnus.utwente.nl>
 *
 * This file is part of the DEIRA project, which is free software; you can redistribute it
 * and/or modify it under the terms of the GNU "Lesser" General Public License v3 as published by the 
 * Free Software Foundation.
 */

package nl.utwente.ewi.hmi.deira.iam.riam;

import java.util.ArrayList;

/**
 * This class is a representation of any expressions in an event generation rule with a binary operator.
 * Boolean expressions are also considered Math expressions.
 * When interpreted it returns a float value corresponding to the expression.
 * 
 * MathExpressions consist of two expressions and an operator.
 * Expr		::= 	(Expr [ + | - | / | * | > | >= | < | <= | == | != | && | || ] Expr)
 *			|		[ #1 | #2 | #3 | #4 ].[ curr | prev ].[ speed | rank | pos ]
 *			|		[ INT | FLOAT ]
 *
 * @version 1.0
 */
public class EventGenerationMathExpression implements EventGenerationExpression {

	EventGenerationExpression expr1 = null;
	/** < The first expression the operator will be executed on. */
	EventGenerationOperator op = null;
	/** < The operator that will be executed on the two expressions. */
	EventGenerationExpression expr2 = null;

	/** < The second expression the operator will be executed on. */

	/**
	 * Constructor.
	 * 
	 * @param expr1
	 *            The first expression.
	 * @param op
	 *            The operator that will be executed over these two expressions.
	 * @param expr2
	 *            The second expression.
	 */
	public EventGenerationMathExpression(EventGenerationExpression expr1,
			EventGenerationOperator op, EventGenerationExpression expr2) {
		this.expr1 = expr1;
		this.op = op;
		this.expr2 = expr2;
	}

	/**
	 * Interprets the expression.
	 * 
	 * @param raceState
	 *            The (current) raceState for which the expression needs to be
	 *            interpreted.
	 * @return The expression value.
	 */
	public float interpret(ArrayList<Participant> raceState) {
		return op.interpret(raceState, expr1.interpret(raceState), expr2
				.interpret(raceState));
	}

	/**
	 * Function to create a String representation of this expression. Mainly for
	 * debugging purposes.
	 * 
	 * @return A String representation of this expression.
	 */
	public String toString() {
		return "(" + expr1.toString() + " " + op.toString() + " "
				+ expr2.toString() + ")";
	}

}
