/* 
 * Copyright (C) 2009 HMI Group, University of Twente.
 *
 * @author Franois L.A. Knoppel <[alias] francois [on server at] knoppel.nl>
 * @author Almer S. Tigelaar <[alias] almer [on server at] gnome.org>
 * @author Danny Oude Bos <[alias] d.oudebos [on server at] utwente.nl>
 * @author Thijs Alofs <[alias] t.alofs [on server at] alumnus.utwente.nl>
 *
 * This file is part of the DEIRA project, which is free software; you can redistribute it
 * and/or modify it under the terms of the GNU "Lesser" General Public License v3 as published by the 
 * Free Software Foundation.
 */

package nl.utwente.ewi.hmi.deira.iam.riam;

import java.util.ArrayList;
import java.util.logging.Logger;

/**
 * This class is a representation of any number, integer or float, in an event
 * generation rule. When interpreted it returns a float value corresponding to
 * the expression.
 */
public class EventGenerationNumberExpression implements
		EventGenerationExpression {
	private static Logger log = Logger.getLogger("deira.ram");
	/** < Logging instance. */
	private float num = 0.0f;

	/** < The number value of this expression. */

	/**
	 * Constructor.
	 * 
	 * @param s
	 *            String indicating the number value of this expression.
	 */
	public EventGenerationNumberExpression(String s) {
		try {
			num = Float.parseFloat(s);
		} catch (NumberFormatException e) {
			log.severe("Could not convert to number: " + s);
		}
	}

	/**
	 * Interprets the expression.
	 * 
	 * @param raceState
	 *            The (current) raceState for which the expression needs to be
	 *            interpreted.
	 * @return The expression value.
	 */
	public float interpret(ArrayList<Participant> raceState) {
		return num;
	}

	/**
	 * Function to create a String representation of this expression. Mainly for
	 * debugging purposes.
	 * 
	 * @return A String representation of this expression.
	 */
	public String toString() {
		return "" + num;
	}

}
