/* 
 * Copyright (C) 2009 HMI Group, University of Twente.
 *
 * @author Franois L.A. Knoppel <[alias] francois [on server at] knoppel.nl>
 * @author Almer S. Tigelaar <[alias] almer [on server at] gnome.org>
 * @author Danny Oude Bos <[alias] d.oudebos [on server at] utwente.nl>
 * @author Thijs Alofs <[alias] t.alofs [on server at] alumnus.utwente.nl>
 *
 * This file is part of the DEIRA project, which is free software; you can redistribute it
 * and/or modify it under the terms of the GNU "Lesser" General Public License v3 as published by the 
 * Free Software Foundation.
 */

package nl.utwente.ewi.hmi.deira.iam.riam;

import java.util.ArrayList;

/**
 * This class is a representation of any requirement (the boolean expression
 * before the ->) in an event generation rule. When interpreted it returns a
 * boolean value corresponding to the expression.
 * 
 * Part of EventGenerationRule.
 * 
 * @version 1.0
 */
public class EventGenerationRequirement {
	EventGenerationExpression expr = null;

	/**
	 * < A (boolean) expression indicating the requirement that needs to be true
	 * for the corresponding event to be generated.
	 */

	/**
	 * Constructor.
	 * 
	 * @param expr
	 *            A (boolean) expression indicating the requirement that needs
	 *            to be true for the corresponding event to be generated.
	 */
	public EventGenerationRequirement(EventGenerationExpression expr) {
		this.expr = expr;
	}

	/**
	 * Interprets the requirement.
	 * 
	 * @param raceState
	 *            The (current) raceState for which the requirement needs to be
	 *            interpreted.
	 * @return Whether the requirement evaluated true or false.
	 */
	public boolean interpret(ArrayList<Participant> raceState) {
		return (expr.interpret(raceState) > 0.0f);
	}

	/**
	 * Function to create a String representation of this requirement. Mainly
	 * for debugging purposes.
	 * 
	 * @return A String representation of this requirement.
	 */
	public String toString() {
		return expr.toString();
	}

}
