/* 
 * Copyright (C) 2009 HMI Group, University of Twente.
 *
 * @author Franois L.A. Knoppel <[alias] francois [on server at] knoppel.nl>
 * @author Almer S. Tigelaar <[alias] almer [on server at] gnome.org>
 * @author Danny Oude Bos <[alias] d.oudebos [on server at] utwente.nl>
 * @author Thijs Alofs <[alias] t.alofs [on server at] alumnus.utwente.nl>
 *
 * This file is part of the DEIRA project, which is free software; you can redistribute it
 * and/or modify it under the terms of the GNU "Lesser" General Public License v3 as published by the 
 * Free Software Foundation.
 */

package nl.utwente.ewi.hmi.deira.iam.riam;

import java.util.ArrayList;

import nl.utwente.ewi.hmi.deira.queue.HREvent;

/**
 * This class is a representation of an event generation rule. When interpreted
 * it returns a RaceEvent if the requirement evaluated true, or null if false.
 * 
 * @version 1.0
 */
public class EventGenerationRule {

	private EventGenerationRequirement req;
	/**
	 * < The requirement that needs to evaluate true for the event to be
	 * generated.
	 */
	private EventGenerationEvent evt;

	/**
	 * < The representation of the event that needs to be generated if the
	 * requirement evaluates true.
	 */

	/**
	 * Constructor.
	 * 
	 * @param req
	 *            The requirement that needs to evaluate true for the event to
	 *            be generated.
	 * @param evt
	 *            The representation of the event to be generated if the
	 *            requirement evaluates true.
	 */
	public EventGenerationRule(EventGenerationRequirement req,
			EventGenerationEvent evt) {
		this.req = req;
		this.evt = evt;
	}

	/**
	 * Interprets the event generation rule.
	 * 
	 * @param newEventID
	 *            A fresh event id to indicate this newly generated event.
	 * @param raceState
	 *            The (current) raceState for which the rule needs to be
	 *            interpreted.
	 * @param startTime
	 *            The system time at which the race itself started.
	 * @return A RaceEvent if the requirement evaluated true, else null.
	 */
	public HREvent interpret(int newEventID, ArrayList<Participant> raceState,
			long startTime) {
		if (req.interpret(raceState)) {
			return evt.interpret(newEventID, raceState, startTime);
		} else
			return null;
	}

	/**
	 * Function to create a String representation of this event generation rule.
	 * Mainly for debugging purposes.
	 * 
	 * @return A String representation of this event generation rule.
	 */
	public String toString() {
		return req.toString() + " -> " + evt.toString();
	}

}
