/* 
 * Copyright (C) 2009 HMI Group, University of Twente.
 *
 * @author Franois L.A. Knoppel <[alias] francois [on server at] knoppel.nl>
 * @author Almer S. Tigelaar <[alias] almer [on server at] gnome.org>
 * @author Danny Oude Bos <[alias] d.oudebos [on server at] utwente.nl>
 * @author Thijs Alofs <[alias] t.alofs [on server at] alumnus.utwente.nl>
 *
 * This file is part of the DEIRA project, which is free software; you can redistribute it
 * and/or modify it under the terms of the GNU "Lesser" General Public License v3 as published by the 
 * Free Software Foundation.
 */

package nl.utwente.ewi.hmi.deira.iam.riam;

import java.util.ArrayList;

/**
 * A participant in the race.
 */
public class Participant {
	private String name;
	/** < Name of the participant */
	private ArrayList<State> states = new ArrayList<State>();

	/** < The current and previous states of the participant */

	/**
	 * Constructor.
	 * 
	 * @param name
	 *            The name of the participant.
	 */
	public Participant(String name) {
		this.name = name;
	}

	/**
	 * Returns the name of the participant.
	 * 
	 * @return The name of the participant.
	 */
	public String getName() {
		return name;
	}

	/**
	 * Returns the current (last added) state of this participant.
	 * 
	 * @return The current (last added) state of this participant.
	 */
	public State getCurrentState() {
		if (states.size() > 0)
			return (State) states.get(states.size() - 1);
		else
			return null;
	}

	/**
	 * Returns the previous (previously added) state of this participant.
	 * 
	 * @return the previous (previously added) state of this participant.
	 */
	public State getPreviousState() {
		if (states.size() > 1)
			return (State) states.get(states.size() - 2);
		else
			return null;
	}

	/**
	 * Adds a new state to the state history of this participant. The added
	 * state will be the participants current state, for as long no new states
	 * are added. The previously current state will now be the previous state,
	 * for as long no new states are added.
	 * 
	 * @param state
	 *            The state to be added to the state history.
	 */
	public void addState(State state) {
		states.add(state);
	}

	/**
	 * Returns the current number of states in the history of this participant.
	 * 
	 * @return The number of states in the history of this participant.
	 */
	public int stateSize() {
		return states.size();
	}

	/**
	 * Returns a String representation of this participant, being the name of
	 * the participant.
	 * 
	 * @return the name of this participant.
	 */
	public String toString() {
		return name;
	}

	public boolean equals(Object o) {
		if (o instanceof Participant) {
			Participant p = (Participant) o;
			if (p.toString().equals(toString()))
				return true;
			else
				return false;
		} else
			return false;
	}

}
