/* 
 * Copyright (C) 2009 HMI Group, University of Twente.
 *
 * @author Franois L.A. Knoppel <[alias] francois [on server at] knoppel.nl>
 * @author Almer S. Tigelaar <[alias] almer [on server at] gnome.org>
 * @author Danny Oude Bos <[alias] d.oudebos [on server at] utwente.nl>
 * @author Thijs Alofs <[alias] t.alofs [on server at] alumnus.utwente.nl>
 *
 * This file is part of the DEIRA project, which is free software; you can redistribute it
 * and/or modify it under the terms of the GNU "Lesser" General Public License v3 as published by the 
 * Free Software Foundation.
 */

package nl.utwente.ewi.hmi.deira.iam.riam;

import java.util.Comparator;
import java.util.logging.Logger;

/**
 * Comparator class that compares participants based on their current position
 * and previous rank. In the case of equal positions, the horse which got a rank
 * first keeps it.
 */
public class ParticipantComparator<T> implements Comparator<T> {

	private static Logger log = Logger.getLogger("deira.ram");

	/** < Logging instance */

	/**
	 * Constructor.
	 */
	public ParticipantComparator() {
	}

	/**
	 * Compares two RaceEvents based on their current importance.
	 * 
	 * @param o1
	 *            First RaceEvent to compare.
	 * @param o2
	 *            Second RaceEvent to compare with.
	 * @returns See Comparator interface, return value is conformant.
	 */
	public int compare(T o1, T o2) {
		if (o1 instanceof Participant && o2 instanceof Participant) {
			int p1 = ((Participant) o1).getCurrentState().getPosition();
			int r1 = ((Participant) o1).getPreviousState().getRank();
			int p2 = ((Participant) o2).getCurrentState().getPosition();
			int r2 = ((Participant) o2).getPreviousState().getRank();
			if (p1 != p2)
				return p2 - p1;
			else
				return r1 - r2;
		} else {
			log.severe("ParticipantComparator: objects [" + o1 + ", " + o2
					+ "] not both of Participant class");
			return 0;
		}
	}

}
