/* 
 * Copyright (C) 2009 HMI Group, University of Twente.
 *
 * @author Franois L.A. Knoppel <[alias] francois [on server at] knoppel.nl>
 * @author Almer S. Tigelaar <[alias] almer [on server at] gnome.org>
 * @author Danny Oude Bos <[alias] d.oudebos [on server at] utwente.nl>
 * @author Thijs Alofs <[alias] t.alofs [on server at] alumnus.utwente.nl>
 *
 * This file is part of the DEIRA project, which is free software; you can redistribute it
 * and/or modify it under the terms of the GNU "Lesser" General Public License v3 as published by the 
 * Free Software Foundation.
 */

package nl.utwente.ewi.hmi.deira.iam.riam;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;

import nl.utwente.ewi.hmi.deira.db.BackgroundInformation;

/**
 * Test class for ParticipantComparator.java
 */
public class ParticipantComparatorTest {

	/**
	 * Prints the information on screen (System.out.println).
	 * 
	 * @param s
	 *            The text to show on screen.
	 */
	private static void log(String s) {
		System.out.println(s);
	}

	/**
	 * The participant comparator is tested on a variety of race states.
	 * 
	 * @param args
	 *            N/A
	 */
	public static void main(String[] args) {

		Comparator<Object> comparator = new ParticipantComparator<Object>();

		ArrayList<Participant> raceState = new ArrayList<Participant>();

		long startTime = System.currentTimeMillis();

		raceState.add(new Participant(BackgroundInformation.PARTICIPANT[0]));
		raceState.add(new Participant(BackgroundInformation.PARTICIPANT[1]));
		raceState.add(new Participant(BackgroundInformation.PARTICIPANT[2]));
		raceState.add(new Participant(BackgroundInformation.PARTICIPANT[3]));

		raceState.get(0)
				.addState(
						new nl.utwente.ewi.hmi.deira.iam.riam.State(startTime,
								0, 0, 1));
		raceState.get(1)
				.addState(
						new nl.utwente.ewi.hmi.deira.iam.riam.State(startTime,
								0, 0, 2));
		raceState.get(2)
				.addState(
						new nl.utwente.ewi.hmi.deira.iam.riam.State(startTime,
								0, 0, 3));
		raceState.get(3)
				.addState(
						new nl.utwente.ewi.hmi.deira.iam.riam.State(startTime,
								0, 0, 4));

		raceState.get(0).addState(new State(0, 100, 0, 0));
		raceState.get(1).addState(new State(0, 100, 0, 0));
		raceState.get(2).addState(new State(0, 100, 0, 0));
		raceState.get(3).addState(new State(0, 100, 0, 0));

		log("Test1: expected result: A, C, E, T");
		Object[] a = raceState.toArray();
		Arrays.sort(a, comparator);
		for (int i = 0; i < a.length; i++) {
			Participant p = (Participant) a[i];
			p.getCurrentState().setRank(i + 1);
			log("Test1-" + i + ": " + p + ": " + p.getCurrentState());
		}

		raceState.get(0).addState(new State(0, 10, 0, 0));
		raceState.get(1).addState(new State(0, 20, 0, 0));
		raceState.get(2).addState(new State(0, 30, 0, 0));
		raceState.get(3).addState(new State(0, 40, 0, 0));

		log("Test2: expected result: T, E, C, A");
		a = raceState.toArray();
		Arrays.sort(a, comparator);
		for (int i = 0; i < a.length; i++) {
			Participant p = (Participant) a[i];
			p.getCurrentState().setRank(i + 1);
			log("Test2-" + i + ": " + p + ": " + p.getCurrentState());
		}

		raceState.get(0).addState(new State(0, 10, 0, 0));
		raceState.get(1).addState(new State(0, 10, 0, 0));
		raceState.get(2).addState(new State(0, 40, 0, 0));
		raceState.get(3).addState(new State(0, 40, 0, 0));

		log("Test3: expected result: T, E, C, A");
		a = raceState.toArray();
		Arrays.sort(a, comparator);
		for (int i = 0; i < a.length; i++) {
			Participant p = (Participant) a[i];
			p.getCurrentState().setRank(i + 1);
			log("Test3-" + i + ": " + p + ": " + p.getCurrentState());
		}

	}

}
