/* 
 * Copyright (C) 2009 HMI Group, University of Twente.
 *
 * @author Franois L.A. Knoppel <[alias] francois [on server at] knoppel.nl>
 * @author Almer S. Tigelaar <[alias] almer [on server at] gnome.org>
 * @author Danny Oude Bos <[alias] d.oudebos [on server at] utwente.nl>
 * @author Thijs Alofs <[alias] t.alofs [on server at] alumnus.utwente.nl>
 *
 * This file is part of the DEIRA project, which is free software; you can redistribute it
 * and/or modify it under the terms of the GNU "Lesser" General Public License v3 as published by the 
 * Free Software Foundation.
 */

package nl.utwente.ewi.hmi.deira.iam.riam;

/**
 * The state of one participant at one moment in time (HRSS tick).
 */
public class State {
	private int position;
	/** < meters from start position, 0-2000m */
	private int speed;
	/** < speed, 0-..km/h */
	private long timestamp;
	/** < timestamp, time since start of the race, in milliseconds */
	private long systemTimestamp;
	/** < timestamp, system time, in milliseconds */
	private int rank;

	/** < rank of the participant, 1-4 */

	/**
	 * Constructor.
	 * 
	 * @param timestamp
	 *            Time since start of the race.
	 * @param position
	 *            Position of the participant in meters from the start.
	 * @param speed
	 *            Speed of the participant in km/h.
	 * @param rank
	 *            Rank of the participant.
	 */
	public State(long timestamp, int position, int speed, int rank) {
		this.systemTimestamp = System.currentTimeMillis();
		this.timestamp = timestamp;
		this.position = position;
		this.speed = speed;
		this.rank = rank;
	}

	/**
	 * Returns the position of the participant in this state.
	 * 
	 * @return The position of the participant in this state.
	 */
	public int getPosition() {
		return position;
	}

	/**
	 * Returns the speed of the participant in this state in km/h.
	 * 
	 * @return The speed of the participant in this state.
	 */
	public int getSpeed() {
		return speed;
	}

	/**
	 * Returns the time of this state, since start of the race, in milliseconds.
	 * 
	 * @return The time of occurrence of this state, since start of the race.
	 */
	public long getTimestamp() {
		return timestamp;
	}

	/**
	 * Returns the time of this state, system time, in milliseconds.
	 * 
	 * @return The time of occurrence of this state, system time.
	 */
	public long getSystemTimestamp() {
		return systemTimestamp;
	}

	/**
	 * Returns the rank of the participant in this state.
	 * 
	 * @return The rank of the participant in this state.
	 */
	public int getRank() {
		return rank;
	}

	/**
	 * Sets the rank of the participant in this state to r.
	 * 
	 * @param r
	 *            the rank of the participant in this state.
	 */
	public void setRank(int r) {
		this.rank = r;
	}

	/**
	 * Returns a String representation of this State.
	 * 
	 * @return A String representation of this state, showing timestamp,
	 *         position, speed and rank.
	 */
	public String toString() {
		return "[" + timestamp + "] " + position + ", " + speed + ", " + rank;
	}

}
