/* 
 * Copyright (C) 2009 HMI Group, University of Twente.
 *
 * @author Franois L.A. Knoppel <[alias] francois [on server at] knoppel.nl>
 * @author Almer S. Tigelaar <[alias] almer [on server at] gnome.org>
 * @author Danny Oude Bos <[alias] d.oudebos [on server at] utwente.nl>
 * @author Thijs Alofs <[alias] t.alofs [on server at] alumnus.utwente.nl>
 *
 * This file is part of the DEIRA project, which is free software; you can redistribute it
 * and/or modify it under the terms of the GNU "Lesser" General Public License v3 as published by the 
 * Free Software Foundation.
 */

package nl.utwente.ewi.hmi.deira.iam.riam;

/**
 * Used by Tokenizer to split up the text into Tokens.
 */
public class Token {

	public static final String RESULTS = "->";
	/** Token definition - RESULTS Token: -> */
	public static final String LPAREN = "(";
	/** Token definition - LPAREN Token: ( */
	public static final String RPAREN = ")";
	/** Token definition - RPAREN Token: ) */

	public static final String GT = ">";
	/** Token definition - GT Token: > */
	public final static String LT = "<";
	/** Token definition - LT Token: < */
	public final static String GTE = ">=";
	/** Token definition - GTE Token: >= */
	public final static String LTE = "<=";
	/** Token definition - LTE Token: <= */
	public final static String EQ = "==";
	/** Token definition - EQ Token: == */
	public final static String NE = "!=";
	/** Token definition - NE Token: != */

	public final static String AND = "&&";
	/** Token definition - AND Token: && */
	public final static String OR = "||";
	/** Token definition - OR Token: || */

	public final static String PLUS = "+";
	/** Token definition - PLUS Token: + */
	public final static String MINUS = "-";
	/** Token definition - MINUS Token: - */
	public final static String MULT = "*";
	/** Token definition - MULT Token: * */
	public final static String DIV = "/";
	/** Token definition - DIV Token: / */

	public final static String SEMICOLON = ";";
	/** Token definition - SEMICOLON Token: ; */
	public final static String COMMA = ",";
	/** Token definition - COMMA Token: , */
	public final static String DOT = ".";
	/** Token definition - DOT Token: . */

	public final static String IDENTIFIER = "IDENTIFIER";
	/** Token definition - IDENTIFIER Token */
	public final static String NUMBER = "NUMBER";
	/** Token definition - NUMBER Token */

	private String content;
	/** String representation of the Token */
	private String type;
	/** Type of the Token */
	private int line;
	/** Line number at which the Token was found */
	private int col;

	/** Column number at which the Token was found */

	/**
	 * Constructor.
	 * 
	 * @param content
	 *            The string representation/content of the Token.
	 * @param type
	 *            The type of the Token.
	 * @param line
	 *            The line at which the Token was found in the text.
	 * @param col
	 *            The column at which the Token was found in the text.
	 */
	public Token(String content, String type, int line, int col) {
		this.content = content;
		this.type = type;
		this.line = line;
		this.col = col;
	}

	/**
	 * Returns the text representation of the Token. For example, the Token type
	 * can be NUMBER, while the string representation is "1.0f".
	 * 
	 * @return The text representation of the Token.
	 */
	public String toString() {
		return content;
	}

	/**
	 * Returns the type of this Token (the types are defined as public final
	 * statics in the Token class). For example, the Token type can be NUMBER,
	 * while the string representation is "1.0f".
	 * 
	 * @return The type of this Token.
	 */
	public String getType() {
		return type;
	}

	/**
	 * Returns the line number at which the Token was found in the tokenized
	 * text.
	 * 
	 * @return The line at which the Token was found in the text file.
	 */
	public int getLine() {
		return line;
	}

	/**
	 * Returns the column number at which the Token was found in the tokenized
	 * text.
	 * 
	 * @return The column at which the Token was found in the text.
	 */
	public int getCol() {
		return col;
	}
}
