package nl.utwente.ewi.hmi.deira.iam.vvciam;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Observable;

/* VERWARRENDE KLASSE NAAM? !Q? */

/**
 * De FileReader klasse leest RoboSoccer data in zet dit om naar Situations. De
 * FileReader werkt alle bestanden genummerd 000000000001.txt t/m
 * xxxxxxxxxxx.txt in een specifieke directory af. Er worden ca. 30 bestanden
 * per seconden uitgelezen, dit zodat een real-time datastream gesimuleerd
 * wordt.<BR>
 * Om het uitlezen van bestanden te starten dient de run() methode als aparte
 * Thread gestart te worden.
 * 
 * @version
 * @author Buursma, Hendriksen, Ten Hoeve, Van der Kooi
 */

public class FileReader extends Observable implements Runnable {
	/** Aantal Situations per seconde */
	public final static int FPS = 30;
	private int moment = 1;
	private int lastFrame = 20000;
	private int millisPerFrame = 1000 / FPS;
	private String directory;

	/**
	 * Creeert een FileReader van RoboSoccer data.
	 * 
	 * @param directory.
	 *            Directory met hierin logfiles. De logfiles zijn genaamd
	 *            xxxxxxxxxxx.txt. beginnend met 00000000001.txt etc.
	 */
	public FileReader(String directory) {
		this.directory = directory;
	}

	/**
	 * Start een loop die alle log bestanden in de directory uitleest.
	 */
	public void run() {
		long start = System.currentTimeMillis();
		for (; moment < lastFrame; moment++) {
			String str_t = directory
					+ ("00000000000".substring(String.valueOf(moment).length()))
					+ moment + ".txt";
			Situation t = readFile(str_t, moment);
			setChanged();
			notifyObservers(t);
			if (t == null) {
				lastFrame = moment;
			}
			try {
				long sleep = (millisPerFrame * moment)
						- (System.currentTimeMillis() - start);
				if (sleep > 0) { // hebben we tijd over, of komen we te kort
									// (dan: betere pc?!?)
					if (sleep > 2000) {
						sleep = 2000;
					}
					Thread.sleep(sleep); // sleepen tot het tijd is voor
											// volgende frame
				}
			} catch (InterruptedException e) {
			} catch (ThreadDeath e) {
			}

		}
	}

	/**
	 * Leest een robosoccer-logefile en creeert een Situation van de huidige
	 * situatie.
	 * 
	 * @param bestand
	 *            uit te lezen robosoccer log.
	 * @param moment
	 *            huidige tijdstip in framenummers(nodig voor create Situation
	 */

	private static Situation readFile(String bestand, int moment) {
		try {
			BufferedReader in = null;
			try {
				in = new BufferedReader(new java.io.FileReader(bestand));
			} catch (FileNotFoundException e) {
				return null;
			}
			FieldObject ball = extractBall(in.readLine());
			FieldObject robots[][] = new FieldObject[2][5];
			for (int i = 0; i < 10; i++) {
				robots[i / 5][i % 5] = extractRobot(in.readLine());
			}
			in.close();
			return new Situation(moment, ball, robots);
		} catch (IOException e) {
			System.out.println("Exception: " + e);
			return null;
		}
	}

	/*
	 * aangepast uit Visualizer package!
	 */
	private static FieldObject extractBall(String str) { // Ball(x,y);1100;900
		int i = str.indexOf(';');
		int i2 = str.indexOf(';', i + 1);
		int x = Math.round(Float.parseFloat(str.substring(i + 1, i2)));
		int y = Math.round(Float.parseFloat(str.substring(i2 + 1)));
		return new FieldObject(x, y);
	}

	/*
	 * aangepast uit Visualizer package!
	 */
	private static FieldObject extractRobot(String str) { // TeamRobot(id,x,y,theta);0;726;276;2.54036e-14
		int x = 0, y = 0;
		int i2 = str.indexOf(';');
		int i = str.indexOf(';', i2 + 1);
		i2 = str.indexOf(';', i + 1);
		x = Math.round(Float.parseFloat(str.substring(i + 1, i2)));
		i = str.indexOf(';', i2 + 1);
		if (str.lastIndexOf(';') == i2) {
			y = Math.round(Float.parseFloat(str.substring(i2 + 1))); // y-waarde
																		// =
																		// einde
																		// van
																		// de
																		// regel
		} else {
			y = Math.round(Float.parseFloat(str.substring(i2 + 1, i))); // na de
																		// y-waarde
																		// komt
																		// nog
																		// theta
																		// waarde
		}
		return new FieldObject(x, y);
	}

	public void close() {
		lastFrame = moment;
	}
}