package nl.utwente.ewi.hmi.deira.iam.vvciam;


import java.awt.Point;
import java.awt.geom.Area;
import java.awt.geom.Ellipse2D;

import nl.utwente.ewi.hmi.deira.iam.vvciam.match.MatchTracker;
import nl.utwente.ewi.hmi.deira.iam.vvciam.text.Text;

/** Deze klasse modelleert een event in een robbosoccer wedstrijd. Deze klasse dient overerft te worden door specifiekere klases.
 * @version
 * @author Buursma, Hendriksen, Ten Hoeve, Van der Kooi
 */

public abstract class RSEvent{
    /** Prioriteit van het Event. */
    protected double priority;
    /** Emotiewaarde van event */
    protected double emotion;
    /** String representatie van het type event.*/
    protected String type;
    /** Commentaar tekst. Initieel <code>null</code> */
    protected Text text;
    /** Naam van team1 . Initieel <code>null</code> */
    protected String team1;
    /** Naam van team2. Initieel <code>null</code> */
    protected String team2;
    /** Naam van robot1. Initieel <code>null</code> */    
    protected String robot1;
    /** Naam van robot2. Initieel <code>null</code> */    
    protected String robot2;
    /** De score als tekst. Initieel <code>null</code> */    
    protected String score;
    /** De locatie van die Event. Initieel <code>null</code> */    
    protected String location;

    /** Construeert een nieuwe Event.
    */  
    public RSEvent(double priority, double emotion, String type, Text text){
        this.priority=priority;
        this.emotion=emotion;
        this.type = type;
        this.text=text;  
	    team1 = "";
	    team2 = "";
        robot1 = "";
        robot2 = "";
        score = "";
        location = "";
    }
    
    /** Geeft de prioriteit van dit Event. */
    public double getPriority(){
        return priority;
    }
    /** Geeft de emotie-waarde van dit Event. */
    public double getEmotion(){
        return emotion;
    }
    /** Geeft de Text van dit Event. */
    public Text getText(){
        return text;
    }
    /** Zet de Text voor dit Event. */
    public void setText(Text text){
        this.text=text;
    }
    /** Geeft het type van dit Event. */
    public String getType(){
        return type;
    }
    /** Geeft de team1 variabele. */
    public String getTeam1() {
    	return team1;
    }
    /** Geeft de team2 variabele. */    
    public String getTeam2() {
    	return team2;
    }
    /** Geeft de robot1 variabele. */    
    public String getRobot1() {
    	return robot1;
    }
    /** Geeft de robot2 variabele. */    
    public String getRobot2() {
    	return robot2;
    }
    /** Geeft de location variabele. */
    public String getLocation() {
    	return location;
    }
    /** Geeft de score variabele. */    
    public String getScore() {
    	return score;
    }
    /** Zet de variabele team1. */
    public void setTeam1(String team1){
 	 	this.team1 = team1;
    }
    /** Zet de variabele team2. */
    public void setTeam2(String team2){
      	this.team2 = team2;
    }
    /** Zet de variabele robot1. */
    public void setRobot1(String robot1){
 	    this.robot1 = robot1;
    }
     /** Zet de variabele robot2. */
    public void setRobot2(String robot2){
 	this.robot2 = robot2;
    }
    /** Zet de variabele location. */    
    public void setLocation(String location) {
    	this.location = location;
    }
    /** Zet de variabele score. */    
    public void setScore(String score) {
    	this.score = score;
    }
    
    /** Geeft een Event terug als dit event gaande is. 
     * Anders: null.
     */
    protected static RSEvent checkEvent(MatchTracker match, java.util.List<Situation> sits, EventGenerator eg){
        return null;
    }
    
    /* Geeft een Area rond het punt p.
    */  
    protected static Area around(Point p, int diameter){
        return new Area(new Ellipse2D.Double(p.getX()-diameter/2, p.getY()-diameter/2, (double)diameter, (double)diameter));
    }
    
    /* Geeft true als de ball in situatie2 minder dan <constante> van de positie in situatie1 verschoven is.
    */
    protected static boolean noBallMove(Situation sit1, Situation sit2){
        return around(sit1.getBall(), 10).contains(sit2.getBall());    
    }
 
    /* Geeft het aantal spelers in de Area.
    */
    protected static int numPlayers(Situation sit, Area ar){
        int r=0;
        for(int a=0; a<2; a++){
            for(int b=0; b<5; b++){
                r = (ar.contains(sit.getRobot(a,b)) ? r+1 : r);
            }
        }    
        return r;
    }
    
}