/* 
 * Copyright (C) 2009 HMI Group, University of Twente.
 *
 * @author Franois L.A. Knoppel <[alias] francois [on server at] knoppel.nl>
 * @author Almer S. Tigelaar <[alias] almer [on server at] gnome.org>
 * @author Danny Oude Bos <[alias] d.oudebos [on server at] utwente.nl>
 * @author Thijs Alofs <[alias] t.alofs [on server at] alumnus.utwente.nl>
 *
 * This file is part of the DEIRA project, which is free software; you can redistribute it
 * and/or modify it under the terms of the GNU "Lesser" General Public License v3 as published by the 
 * Free Software Foundation.
 */

package nl.utwente.ewi.hmi.deira.iam.vvciam;


import java.util.logging.Logger;

import nl.utwente.ewi.hmi.deira.generic.DeiraThread;
import nl.utwente.ewi.hmi.deira.generic.Module;
import nl.utwente.ewi.hmi.deira.iam.vvciam.match.MatchTracker;
import nl.utwente.ewi.hmi.deira.iam.vvciam.visualizer.Visualizer;

public class VirtueleVoetbalCommentatorAnalysis extends DeiraThread implements Module
{

	private static Logger log = Logger.getLogger("deira.vvciam"); /**< Logging instance */
	private String gamepath; /**< The path to the game to analyze */
    private String matchFile;
    private MatchTracker matchtracker;
    private Thread inputstreamthread;
    private nl.utwente.ewi.hmi.deira.iam.vvciam.FileReader inputstream;
    private nl.utwente.ewi.hmi.deira.iam.vvciam.visualizer.Visualizer visualizer;
	private nl.utwente.ewi.hmi.deira.iam.vvciam.EventGenerator eventgenerator;
	private boolean work = true;
	public boolean startGame = false;

	/**
	 */
	public VirtueleVoetbalCommentatorAnalysis(nl.utwente.ewi.hmi.deira.mmm.MMM mmm)
	{
//		super("DEIRA VVCAM");
		
		//=-- Set essential parameters
		this.gamepath = "..\\etc\\VVC\\wedstrijden\\wedstrijd02\\";
		this.matchFile = this.gamepath+"\\MatchFile.txt";
		
		//=-- Initialize rest of object (visualizer etc.)
		init();
		this.setMmm(mmm);
	}
	
	
	/**
	 * @param gamepath
	 * @param matchFile
	 */
	public VirtueleVoetbalCommentatorAnalysis(String gamepath, String matchFile, nl.utwente.ewi.hmi.deira.mmm.MMM mmm)
	{
//		super("DEIRA VVCAM");
		
		//=-- Set essential parameters
		this.gamepath = gamepath;
		if (matchFile != null && !matchFile.equals("")) { this.matchFile = matchFile; } 
		else { this.matchFile = this.gamepath+"\\MatchFile.txt"; }
		
		//=-- Initialize rest of object (visualizer etc.)
		init();
		this.setMmm(mmm);

	}

	/**
	 * Starts the game and starts the module receiving data.
	 */
	public void run() {
		
		//=-- Wait for all threads to be ready TODO: Beter een signaaltje van alle threads dan arbitrary wait.
    	log.info("VVCAM: Pausing to let all threads start");
		try { synchronized(this) {this.wait(1000);} } catch (InterruptedException e) {}
		
		//=-- Wait for send ready to go!
	    try {
	    	log.info("VVCAM: Waiting for permission to start the game (from OM)");
	    	while (!this.startGame ) {
	    		synchronized(this) {
	    			this.wait(1000);
	    		}
	    	}
	    } catch (InterruptedException e) {}
	    
	    	log.info("VVCAM: Got Permission: Starting!");
	    	
			//=-- Start the race!
	    	startGame();
	}

	
	private void startGame()
	{
		//=-- Start filereading
        inputstreamthread = new Thread(this.inputstream, "DEIRA-VVC Inputstream");
        inputstreamthread.start(); 

	    while (work && inputstreamthread.isAlive()) {
			try { synchronized(this) {this.wait(1000);} } catch (InterruptedException e) {} //Live while necessary
	    }
	}

	public void init() {
		//=-- Initialize file input stream
        this.inputstream = new nl.utwente.ewi.hmi.deira.iam.vvciam.FileReader(this.gamepath);

        //=-- Initialize matchtracker, visualizer and eventgenerator
        try {
	        	this.matchtracker = new MatchTracker(this.matchFile);
	        } catch(Exception e){
	        	log.severe("VVCAM: Error creating Match-tracking object!: "+e);
	        }
	    this.visualizer 		= new Visualizer(this.matchtracker);
	    this.eventgenerator 	= new EventGenerator(this.matchtracker); 
	    
	    //=-- Set visualizer parameters
        this.visualizer.setSize(640, 410);
	    
	    //=-- Add matchtracker and visualizer as observers of the inputstream 
	    this.inputstream.addObserver(this.matchtracker);
	    this.inputstream.addObserver(this.visualizer);
	    this.inputstream.addObserver(this.eventgenerator);
	    
	}

	/**
	 * Method to destroy this thread. Sets this.work to false.  
	 */
	public void close()
	{
		if (eventgenerator != null) {this.inputstream.deleteObservers();}
		if (visualizer != null) {visualizer.dispose();}
		if (this.inputstream != null) {this.inputstream.close();}
		this.work = false;
	}


	public nl.utwente.ewi.hmi.deira.mmm.MMM getMmm()
	{
		return eventgenerator.getMmm();
	}


	public void setMmm(nl.utwente.ewi.hmi.deira.mmm.MMM mmm)
	{
		eventgenerator.setMmm(mmm);
	}
	
	public String getModuleName()
	{ return "VVCIAM";	 }
	
}
