package nl.utwente.ewi.hmi.deira.iam.vvciam.speech;
import nl.utwente.ewi.hmi.deira.iam.vvciam.EmotionGenerator;
import nl.utwente.ewi.hmi.deira.iam.vvciam.MatchKickOffEvent;
import nl.utwente.ewi.hmi.deira.iam.vvciam.PenaltyKickMissEvent;
import nl.utwente.ewi.hmi.deira.iam.vvciam.RSEvent;
import nl.utwente.ewi.hmi.deira.iam.vvciam.match.MatchTracker;
import nl.utwente.ewi.hmi.deira.iam.vvciam.text.Text;
import nl.utwente.ewi.hmi.deira.iam.vvciam.text.TextGenerator;

import com.sun.speech.freetts.Voice;
import com.sun.speech.freetts.VoiceManager;

/** Deze klasse verzorgt de omzetting naar (synthetische) spraak. <br>
 * In deze klasse wordt de tekst die
 * door de TextGenerator is geselecteerd omgezet
 * in (synthetische) spraak.<BR>
 * Om deze klasse actief te laten worden dient deze als Thread gestart
 * te worden middels de run() methode.<BR>
 * In deze aparte Thread wordt constant gekeken of in de SpeechQueue 
 * een RSEvent is om te becommentatiseren. Als er een RSEvent is,
 * dan wordt deze middels de TextGenerator omgezet in tekst
 * en vervolgens door deze klasse omgezet in spraak.
 * @version
 * @author Buursma, Hendriksen, Ten Hoeve, Van der Kooi
 */
 public class SpeechGenerator extends Thread{
 
 private SpeechQueue queue;
 private TextGenerator tg;
 private EmotionGenerator eg;
 private Voice voice;
 private MatchTracker match;
 private double speechEmotion;
 private boolean stopped;
 private boolean goalUnderCons;
 private boolean freeball;
 private boolean penalty;
 private boolean notStarted;
 private boolean penaltyTaken;
 private boolean goalkick;
 private boolean boring;
 
 /**
  * Constuctor, instantieert de SpeechQueue.
  */
 public SpeechGenerator(SpeechQueue queue, TextGenerator tg, EmotionGenerator eg, MatchTracker m) {
  this.tg = tg;
  this.eg = eg;
  this.queue = queue;
  this.match=m; //nodig voor bijhouden goals.
  makeVoice();
  stopped = false;
  goalUnderCons = false;
  freeball = false;
  penalty = false;
  notStarted = false;
  penaltyTaken = false;
  goalkick = false;
  boring = false;
 }
 
 private void makeVoice() {
	  VoiceManager voiceManager = VoiceManager.getInstance();
	  Voice[] voices = voiceManager.getVoices();
	  int choose = (int) (Math.random() * 2) + 1;
  	  voice = voices[choose];
	  voice.allocate();
	  introduce();
 }
 
 private void introduce() {
 	voice.speak("Hello");
 	voice.speak("my name is " + voice.getName());
 }
 	
 /*private void printStatus() {
 	System.out.println("pitch: " + voice.getPitch() + ", pitch range: " + voice.getPitchRange() + ", pitch shift: " + voice.getPitchShift());
 	System.out.println("rate: " + voice.getRate());
 	System.out.println("style: " + voice.getStyle());
 	System.out.println("volume: " + voice.getVolume());
 	System.out.println("speech emotion: " + speechEmotion);
 }*/
 
 private void setSpeechProperties(RSEvent e) {
 	double matchEmotion = eg.getEmotion();
 	double eventEmotion = e.getEmotion();
 	speechEmotion = (matchEmotion + eventEmotion) *2  + 0.05;
 	if (speechEmotion > 0.9 && speechEmotion < 1.45) {
 		voice.setRate(new Float(150 * speechEmotion));
 	} else if (speechEmotion < 0.9) {
 		voice.setRate(135);
 	} else {
 		voice.setRate(170);
 	}
 	if (speechEmotion > 0.95 && speechEmotion < 1.3) {
 		voice.setPitch(new Float(100 * speechEmotion));
 	} else if (speechEmotion < 0.95) {
 		voice.setPitch(95);
 	} else {
 		voice.setPitch(130);
 	}
 	if (speechEmotion > 0.91 && speechEmotion < 1.45) {
 		voice.setPitchRange(new Float(11 * speechEmotion));
 	} else if (speechEmotion < 0.91) {
 		voice.setPitchRange(10);
 	} else {
 		voice.setPitchRange(15);
 	}
 	if ( speechEmotion > 0.93 && speechEmotion < 1.2) {
 		voice.setPitchShift(new Float(0.7 * speechEmotion));
 	} else if (speechEmotion < 0.93) {
 	  voice.setPitchShift(new Float(0.65));
 	} else {
 		voice.setPitchShift(new Float(0.9));
 	}
 }

 /**
  * Zet de tekst in een event om in spraak
  * Hiervoor wordt de meest relevant tekst uit de 
  * SpeechQueue gehaald, de hieraan verbonden tekst
  * wordt omgezet in spraak
  * Hierbij wordt rekening gehouden met de emotiewaarde
  * van het event en van het programma
  * Als er als laatste event een goal heeft plaatsgevonden, 
  * worden alle events weggegooid totdat er een kick-off of
  * een goal kick binnenkomt.
  */
 private void textToSpeech(RSEvent e) {
   boolean speak = true;
   if (goalUnderCons &! (e.getType().equals("kick-off") || e.getType().equals("goaldisallowed"))) {
   	// Dit event gooien we weg omdat er net een doelpunt gemaakt is en nog moet worden gekeken of deze is goedgekeurd.
   	speak = false;
   } else if (freeball &! (e.getType().equals("freeballtaken"))) {
   	// Dit event gooien we weg omdat de robots nog bezig zijn hun posities voor de freeball in te nemen.
   	speak = false;
   } else if (goalkick &! (e.getType().equals("goalkicktaken"))) {
   	// Dit event gooien we weg omdat de robots nog bezig zijn hun posities voor de goalkick in te nemen.
   	speak = false;
   } else if (penalty &! (e.getType().equals("penaltytaken"))) {
   	// Dit event gooien we weg omdat de robots nog bezig zijn hun posities voor de penalty in te nemen.
   	speak = false;
   } else if (boring && (e.getType().equals("boring"))) {
   	// Je wilt niet twee keer achterelkaar een boring event
   	speak = false;
   } else if (notStarted &! (e.getType().equals("kick-off"))) {
   	// Dit event gooien we weg omdat een wedstrijd alleen maar kan beginnen met een aftrap.
   	speak = false;
   } else if (penaltyTaken &! (e.getType().equals("shotontarget") || e.getType().equals("worthlesspenaltykick")
   		 || e.getType().equals("goal") || e.getType().equals("owngoal"))) {
   	// Dit event gooien we weg omdat er op dit moment een penalty genomen wordt.
   	speak = false;
   } if (e.getType().equals("end") || e.getType().equals("enddraw")) {
   	speak =true;
   }
   if (speak) {
       	if (notStarted && e.getType().equals("kick-off")) {
       		MatchKickOffEvent mke = new MatchKickOffEvent();
       		mke.setRobot1(e.getRobot1());
       		e = mke;
       		notStarted = false;
       	} else if (penaltyTaken && (e.getType().equals("shotontarget"))) {
       		e = new PenaltyKickMissEvent();
       		penaltyTaken = false;
        }
       	Text t = tg.getText(e, eg.getEmotion());
    	e.setText(t);
    	t.setTeam1(e.getTeam1());
    	t.setTeam2(e.getTeam2());
    	t.setRobot1(e.getRobot1());
    	t.setRobot2(e.getRobot2());
    	t.setLocation(e.getLocation());
    	t.setScore(e.getScore());
    	setSpeechProperties(e);
    	System.out.println(t.getText());
    	voice.speak(t.getText());
    }
    if(!goalUnderCons && (e.getType().equals("goal") || e.getType().equals("owngoal"))){       //scoreboard
        match.goal(e.getTeam1()==match.getTeam1().getName()? 0 : 1); 
    }
   
    if (e.getType().equals("kick-off")) {
   	goalUnderCons = false;
    } else if (e.getType().equals("goal")) {
       	goalUnderCons = true;
       	penaltyTaken = false;
    } else if (e.getType().equals("owngoal")) {
       	goalUnderCons = true;
       	penaltyTaken = false;
    } else if (e.getType().equals("goaldisallowed")) {
    	goalUnderCons = false;
    	goalkick = true;
    } else if (e.getType().equals("freeball")) {
   	    freeball = true;
    } else if (e.getType().equals("goalkick")) {
   	    goalkick = true;
    } else if (e.getType().equals("goalkicktaken")) {
   	    goalkick = false;
    } else if (e.getType().equals("freeballtaken")) {
   	    freeball = false;
    } else if (e.getType().equals("penaltykick")) {
   	    penalty = true;
    } else if (e.getType().equals("penaltykicktaken")) {
   	    penalty = false;
   	    penaltyTaken = true;
    } else if (e.getType().equals("worthlesspenaltykick")) {
   	    penaltyTaken = false;
    } else if (e.getType().equals("start")) {
   	    notStarted = true;
    } else if (e.getType().equals("boring")) {
   	    boring = true;
    }
 }
 
 private void stopSpeech() {
 	voice.deallocate();
 	stopped = true;
 }
 
 /** Start de SpeechGenerator Thread. <BR>
  *  Deze Thread zal constant de VSEvents in de SpeechQueue proberen uit te lezen.
  *
  */
 public void run() {
 	RSEvent e;
 	while (!stopped || queue.getLength() > 0) {
 		synchronized (queue) {
 		  e = null;
        	  while (queue.getLength() == 0 && !stopped) {
           		try {
              			queue.wait();
           		} catch (InterruptedException ie) {	}
        	  }
        	  if (queue.getLength() > 0) {
           			e = queue.giveEvent();
        	  }
     	       }
   	       if (e != null) {
     	       	textToSpeech(e);
   	       }
   	}
   	stopSpeech();
  }
}
