package nl.utwente.ewi.hmi.deira.iam.vvciam.speech;

import java.util.ArrayList;
import java.util.List;

import nl.utwente.ewi.hmi.deira.iam.vvciam.RSEvent;



/**
 * De klasse SpeechQueue
 * In deze klasse worden de events die in
 * spraak moeten worden omgezet opgeslagen
 * De SpeechQueue zet de events die nieuw 
 * binnenkomen op de juiste plaats in de lijst
 * Verder zorgt de SpeechQueue er voor dat het
 * meest relevante event van dat moment op de
 * eerste positie van de lijst staat
 * Zodra de Speechgenerator een event vraagt
 * zal dit eerste element worden gegeven
 * 
 * Let op de synchronisatie van functies op deze queue.
 * @version
 * @author Buursma, Hendriksen, Ten Hoeve, Van der Kooi
 */

public class SpeechQueue {

	private List<RSEvent> events;
	
	/**
	 * Constructor, instantieert de lijst
	 */
	public SpeechQueue() {
		events = new ArrayList<RSEvent>();
	}
	
	/**
	 * zet een Event in de queue
	 * Hierbij wordt geordend op prioriteit
	 * Deze functie synchronized automatisch op de huidige instantie.
	 * @param event het Event dat in de Queue gezet wordt
	 */
	public void enqueue(RSEvent event) {
		synchronized(this) {
			Double priority = event.getPriority();
			int index = 0;
			while (events.size() > index && events.get(index).getPriority() > priority) {
				index ++;
			}
			events.add(index, event);
			notifyAll();
		}
	}
	
	/**
	 * update de queue, zodat het meest relevante
	 * event vooraan de queue komt. Verder worden 
	 * verouderde events verwijderd uit de lijst
	 */
	public void update(RSEvent e) {
		int i = 0;
		while (i < events.size()) {
			if (e.getType().equals(events.get(i).getType())) {
				events.remove(i);
			} else if (events.get(i).getType().equals("ballpossession") || events.get(i).getType().equals("pass")) {
				events.remove(i);
			} else {
				i++;
			}
		}
	}
	
	/**
	 * Retourneert het meest relevante event
	 * hiervoor wordt de lijst eerst geupdate
	 * en komt het meest relevante event op de 
	 * eerste plaats in de queue.
	 * Voor aanroep van deze functie synchroniseren op deze instantie van SpeechQueue.
	 */
	public RSEvent giveEvent() {
		RSEvent e = events.remove(0);
		update(e);
		return e;
	}
	
	/**
	 * Retourneert de lengte van de queue
	 */
	public int getLength() {
		return events.size();
	}
}
