package nl.utwente.ewi.hmi.deira.iam.vvciam.text;

/**
 * Een tekstregel horend bij een bepaald event.
 * @version
 * @author Buursma, Hendriksen, Ten Hoeve, Van der Kooi
 */

public class Text {
  
  private String text;
  private String eventType;
  private double minEmotion;
  private double maxEmotion;
  
  /**
  * Constructor van de klasse Text.
  * Deze maakt een nieuwe Text aan. De timesUsed wordt genitialiseerd op 0 en de andere variabelen worden
  * genitialiseerd op de meegegeven parameters.
  * @param text de string-representatie van de tekst
  *        type het eventtype waar de tekstregel bij hoort
  *        minEmotion, maxEmotion de respectievelijk minimale en maximale emotiewaarde waaraan de emotiewaarde
  *                               van een event moet voldoen, om deze tekstregel eraan te kunnen koppelen.
  */
  public Text(String text, String type, double minEmotion, double maxEmotion) {
    this.text = text;
    eventType = type;
    this.minEmotion = minEmotion;
    this.maxEmotion = maxEmotion;
  }
  
  /** Geeft een exacte kopie van de instantie waarop het wordt aangeroepen. <BR>
   *  Wijzigingen aan de kopie hebben geen invloed op het orgineel.
  */
  public Object clone() {
  	String textclone = text;
  	return new Text(textclone, eventType, minEmotion, maxEmotion);
  }
  
  /**
   * Levert de string-representatie van de tekst op.
   */
  public String getText() {
    return text;
  }
  
  /**
   * Levert het eventtype van de tekst op.
   */
  public String getEventType() {
    return eventType;
  }
  
  /**
   * Levert de emotiewaarde op die een event minimaal moet hebben om deze tekstregel eraan te kunnen koppelen.
   */
  public double getMinEmotion() {
    return minEmotion;
  }
  
  /**
   * Levert de emotiewaarde op die een event maximaal mag hebben om deze tekstregel eraan te kunnen koppelen.
   */
  public double getMaxEmotion() {
    return maxEmotion;
  }

  /** Vervangt de TEAM1 tag in de tekst door de gegeven eigenlijke teamnaam.<BR>
   * @param team1 teamnaam.
   */
  public void setTeam1(String team1) {
    String replace = "<TEAM1>";
    for (int i = 0; i < text.length(); i++) {
      if (text.startsWith(replace,i)) {
        text = text.substring(0,i).concat(team1).concat(text.substring((i+replace.length()),(text.length())));
      }
    }
  }
  
  /** Vervangt de TEAM2 tag in de tekst door de gegeven eigenlijke teamnaam.<BR>
   * @param team2 teamnaam.
   */
  public void setTeam2(String team2) {
    String replace = "<TEAM2>";
    for (int i = 0; i < text.length(); i++) {
      if (text.startsWith(replace,i)) {
        text = text.substring(0,i).concat(team2).concat(text.substring((i+replace.length()),(text.length())));
      }
    }
  }
  
  /** Vervangt de ROBOT1 tag in de tekst door de gegeven eigenlijke robotnaam.<BR>
   * robot1 is in elke tekst de belangrijkste robot (maker van doelpunt, nemer van vrije trap, doeltrap e.d.)
   * @param robot1 Naam van de robot.
   */
  public void setRobot1(String robot1) {
    String replace = "<ROBOT1>";
    for (int i = 0; i < text.length(); i++) {
      if (text.startsWith(replace,i)) {
        text = text.substring(0,i).concat(robot1).concat(text.substring((i+replace.length()),(text.length())));
      }
    }
  }
  
 /** Vervangt de ROBOT2 tag in de tekst door de gegeven eigenlijke robotnaam.<BR>
   * Een tweede robot (b.v. tweede robot bij een vrije bal, aangever van een doelpunt, gever van een pass)
   * @param robot1 Naam van de robot.   
   */
  public void setRobot2(String robot2) {
    String replace = "<ROBOT2>";
    for (int i = 0; i < text.length(); i++) {
      if (text.startsWith(replace,i)) {
        text = text.substring(0,i).concat(robot2).concat(text.substring((i+replace.length()),(text.length())));
      }
    }
  }
 
  /** Vervangt de LOCATION tag in de tekst door de gegeven eigenlijke lokatie.<BR>
   * @param location  Plaats van het event (linkerbovenhoek, rechteronderhoek o.i.d.)
   */
  public void setLocation(String location) {
    String replace = "<LOCATION>";
    for (int i = 0; i < text.length(); i++) {
      if (text.startsWith(replace,i)) {
        text = text.substring(0,i).concat(location).concat(text.substring((i+replace.length()),(text.length())));
      }
    }
  }
  
  /** Vervangt de SCORE tag in de tekst door de gegeven eigenlijke score.<BR>
   * @param score De huidige stand in tekst, bijvoorbeeld: "1 nil".
   */
  public void setScore(String score) {
    String replace = "<SCORE>";
    for (int i = 0; i < text.length(); i++) {
      if (text.startsWith(replace,i)) {
        text = text.substring(0,i).concat(score).concat(text.substring((i+replace.length()),(text.length())));
      }
    }
  }
  
}