package nl.utwente.ewi.hmi.deira.iam.vvciam.text;



import java.io.BufferedReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import nl.utwente.ewi.hmi.deira.iam.vvciam.RSEvent;


/**
 * Een tekstgenerator die bij een gegeven event een geschikte tekst zoekt
 * @version
 * @author Buursma, Hendriksen, Ten Hoeve, Van der Kooi
 */

public class TextGenerator {
  
  private List<Text> availableTexts;
  private List<Text> recentlyUsed;
  
  /**
  * Constructor van de klasse TextGenerator. <BR>
  * Deze constructor leest een lijst van alle mogelijke teksten in uit een gegeven bestand.
  *
  * @param filename Bestand met alle commentaar teksten.
  * @throws IOException Wanneer er IOExceptions optreden bij het uitlezen van filename.
  */
  public TextGenerator(String filename) throws IOException {
    availableTexts = new ArrayList<Text>();
    recentlyUsed = new ArrayList<Text>();
    readFirst(filename);
  }
  
  private void readFirst(String filename) throws IOException{
            BufferedReader in = null;
            in = new BufferedReader(new java.io.FileReader(filename)); //gepaste plek voor dit bestand?   
            String gelezen = in.readLine();
            while(gelezen != null){
    	        int seperator = gelezen.indexOf("|");
    	        int tweedeseperator = gelezen.indexOf("|",seperator+1);
    	        int derdeseperator = gelezen.indexOf("|",tweedeseperator+1);
    	        String text = gelezen.substring(0,seperator);	
    	        String type = gelezen.substring(seperator+1,tweedeseperator);
    	        double minemotie = Double.valueOf(gelezen.substring(tweedeseperator+1,derdeseperator));
    	        double maxemotie = Double.valueOf(gelezen.substring(derdeseperator+1));
    	        this.addText(text,type,minemotie,maxemotie);	
    	        gelezen = in.readLine();
       	    }    	
    }  
  /** Voegt een Text toe aan de lijst beschikbare teksten.
   * @param text de string-representatie van de tekst
   *        type het eventtype waar de tekstregel bij hoort
   *        minEmotion, maxEmotion de respectievelijk minimale en maximale emotiewaarde waaraan de emotiewaarde
   *                               van een event moet voldoen, om deze tekstregel eraan te kunnen koppelen.
   */
  public void addText(String text, String type, double minEmotion, double maxEmotion) {
  	Text t = new Text(text, type, minEmotion, maxEmotion);
  	availableTexts.add(t);
  }  	
  
  /**
  * Methode om een geschikte tekst bij het gegeven event uit te zoeken.
  * Eerst wordt er gekeken welke teksten qua emotiewaarde en type berhaupt bij het meegegeven event passen
  * en deze teksten worden in een nieuwe lijst geplaatst. Nadat er een tekst gekozen is, wordt deze lijst weer
  * geleegd. Vervolgens wordt er willekeurig, maar ook kijkend naar het aantal keren dat tekstregels eerder al
  * gekozen zijn, een tekst uitgezocht.
  * Daarna wordt de gekozen tekst gekoppeld aan het event en wordt de timesUsed van de tekst opgehoogd en 
  * ten slotte wordt het event in de wachtrij geplaatst van de SpeechQueue.
  */
  public Text getText(RSEvent event, double emotion) {
    Text text = null;
    String eventType = event.getType();
    int totalTimesUsed = 0;
    List<Text> possibleTexts = new ArrayList<Text>();
    for (int i = 0; i < availableTexts.size(); i++) {
      if (eventType.equals(availableTexts.get(i).getEventType()) && 
         availableTexts.get(i).getMinEmotion() <= emotion && emotion <= availableTexts.get(i).getMaxEmotion()) {
         possibleTexts.add(availableTexts.get(i));
      }
    }
    if (possibleTexts.size() > 0) {
      List<Integer> notRecentlyUsed = new ArrayList<Integer>();
      for (int i = 0; i < possibleTexts.size(); i++) {
      	if (!recentlyUsed.contains(possibleTexts.get(i))){
      		notRecentlyUsed.add(i);
        }
      }
      int willekeurig = 0;
      if (notRecentlyUsed.size() > 0) {
      	int pos = (int) (Math.random()*notRecentlyUsed.size());
      	willekeurig = notRecentlyUsed.get(pos);
      }	
      else {
      	willekeurig = (int) (Math.random()*possibleTexts.size());
      }
      text = possibleTexts.get(willekeurig);
      possibleTexts.clear();
      recentlyUsed.add(text);
      if (recentlyUsed.size() > 10) {
      	recentlyUsed.remove(0);
      }
    } else {
      	// debugregel, het is per definitie onmogelijk dat hij hier komt als voor alle events teksten zijn bedacht.
      	System.out.println("Voor dit event hebben we nog geen tekst bedacht");
    }
    return (Text) text.clone();
  }
  
}