package nl.utwente.ewi.hmi.deira.iam.vvciam.visualizer;


import java.awt.Color;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.util.Observable;
import java.util.Observer;

import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;

import nl.utwente.ewi.hmi.deira.iam.vvciam.Situation;
import nl.utwente.ewi.hmi.deira.iam.vvciam.match.MatchTracker;


/** Visualizer creert een venster waarin een robosoccer-spel wordt weergegeven. Tevens creert deze klasse een Score instantie.
 * Deze klasse dient als Observer aan een analyzer.FileReader object gekoppeld te zijn. Dat Observable object voorziet de visualizer de actuele spelsituatie.
 * @version
 * @author Buursma, Hendriksen, Ten Hoeve, Van der Kooi
 */
public class Visualizer extends JFrame implements Observer{

    private PlayField playField;
    private JLabel playerList;
	private Score scoreBoard;

    /** Creeert een nieuwe Visualizer instantie.
     * @param m Hieruit worden de teamgegevens gelezen.
     * @require m != null
    */	            
    public Visualizer(MatchTracker m){
        setTitle("Robosoccer-log Visualizer");
        setLocation(400,100);
        setVisible(true);        
        playField=new PlayField();
        JLabel team1 =new JLabel(m.getTeam1().getName());
        team1.setForeground(Color.blue);
        JLabel team2 =new JLabel(m.getTeam2().getName());
        team2.setForeground(Color.red);
        JPanel teams = new JPanel(new GridLayout(4,1));
        for(int i=0; i<2; i++){
            JPanel team=new JPanel(new GridLayout(5,1));
            for(int j=0; j< 5; j++){
                JLabel player=new JLabel(j + " - " + m.getTeam(i).getRobot(j).getName());
                player.setForeground(i==0 ? Color.blue : Color.red);
                team.add(player);
            }
            teams.add(i==0 ? team1 : team2);
            teams.add(team);
        }
        playerList=new JLabel("test\nblabla");
        setLayout(new FlowLayout());
        getContentPane().add(playField);
        getContentPane().add(teams);
        playField.setLocation(0,0);
        scoreBoard = new Score(m);
        m.addObserver(scoreBoard);
        setDefaultCloseOperation(EXIT_ON_CLOSE);
    }
    
    /** Methode dient alleen aangeroepen te worden via het Observer/Observable pattern, door een analyzer.FileReader instantie.
     * @param arg een instantie van Situation met de actuele spelsituatie.
     */
    public void update(Observable o, Object arg){
        if(arg instanceof Situation){
            newSituation((Situation) arg);
        }    
    }
    
    /** Deze methode zorgt ervoor dat het scorebord gerepaint wordt bij een nieuwe situatie
     * @param situation de nieuwe situatie
     */
    private void newSituation(Situation situation){
        playField.setSituation(situation);
        playField.repaint();    
    }
    
    public void dispose() {
    	scoreBoard.dispose();
    	super.dispose();
    }
    
}