/* 
 * Copyright (C) 2009 HMI Group, University of Twente.
 *
 * @author Franois L.A. Knoppel <[alias] francois [on server at] knoppel.nl>
 * @author Almer S. Tigelaar <[alias] almer [on server at] gnome.org>
 * @author Danny Oude Bos <[alias] d.oudebos [on server at] utwente.nl>
 * @author Thijs Alofs <[alias] t.alofs [on server at] alumnus.utwente.nl>
 *
 * This file is part of the DEIRA project, which is free software; you can redistribute it
 * and/or modify it under the terms of the GNU "Lesser" General Public License v3 as published by the 
 * Free Software Foundation.
 */

package nl.utwente.ewi.hmi.deira.mmm;

import java.util.HashMap;
import java.util.Map.Entry;

import nl.utwente.ewi.hmi.deira.queue.Event;

public class EmotionalTagger {
	protected HashMap<String, Double> generatedEmotions = new HashMap<String, Double>();
	protected int duration = 0;
	protected boolean useImportance = true;
	protected double importanceFactor = 1;

	public EmotionalTagger() {
	}

	/**
	 * Processes the event. Sets this.generatedEmotions and this.duration. Meant
	 * to be overloaded.
	 * 
	 * @param event
	 */
	public void processEvent(Event event) {
		// =-- Do nothing
	}

	/**
	 * Gets the emotions generated for the event with which this tagger was
	 * created
	 * 
	 * @return
	 */
	public HashMap<String, Double> getGeneratedEmotions() {
		return generatedEmotions;
	}

	/**
	 * Gets the duration generated for the event with which this tagger was
	 * created
	 * 
	 * @return
	 */
	public int getDuration() {
		return duration;
	}

	/**
	 * Applies the importancefactor to the emotions
	 */
	public void applyImportance() {
		// =-- Check whether we should be doing this at all
		if (this.useImportance) {

			// =- Initialize storage map
			HashMap<String, Double> updatedemotions = new HashMap<String, Double>();

			// =-- Iterate previously generated emotions map
			for (Entry<String, Double> currentemotion : generatedEmotions
					.entrySet()) {
				updatedemotions.put(currentemotion.getKey(), currentemotion
						.getValue()
						* importanceFactor);
			}

			// =-- Update previously generated emotions
			generatedEmotions = updatedemotions;
		}
	}

}
