/* 
 * Copyright (C) 2009 HMI Group, University of Twente.
 *
 * @author Franois L.A. Knoppel <[alias] francois [on server at] knoppel.nl>
 * @author Almer S. Tigelaar <[alias] almer [on server at] gnome.org>
 * @author Danny Oude Bos <[alias] d.oudebos [on server at] utwente.nl>
 * @author Thijs Alofs <[alias] t.alofs [on server at] alumnus.utwente.nl>
 *
 * This file is part of the DEIRA project, which is free software; you can redistribute it
 * and/or modify it under the terms of the GNU "Lesser" General Public License v3 as published by the 
 * Free Software Foundation.
 */

package nl.utwente.ewi.hmi.deira.mmm;

import java.util.logging.Logger;

import nl.utwente.ewi.hmi.deira.queue.Event;
import nl.utwente.ewi.hmi.deira.queue.HREvent;

/**
 * A class that generates emotions for Horse Race Events
 * 
 * @author Dual Infinity
 */
public class HREmotionalTagger extends EmotionalTagger {

	private final int uur = 1000 * 60 * 60;
	private static Logger log = Logger.getLogger("deira.hret");

	/** < Logging instance */

	public HREmotionalTagger() {
		super();
		useImportance = true;
	}

	public void processEvent(Event event) {
		// =-- Input checking
		HREvent raceevent;
		if (event instanceof HREvent) {
			raceevent = (HREvent) event;
		} else {
			return;
		}

		// =-- Variable initialization
		importanceFactor = (raceevent.getImportance() / 5.0);
		int duration = 1;
		double tension = 0.001;
		double surprise = 0.001;
		double amusement = 0.001;
		double pity = 0.001;

		String type = raceevent.getType();
		String lbl = raceevent.getLabel();

		// the position of the first horse in the race is in general very
		// important for the tension level of the race
		if (type.equals("POSITION")) {
			if (lbl.equals("FIRSTTURN")) {
				tension = 0.1;
				surprise = 0.0;
				amusement = 0.0;
				pity = 0.0;
				duration = uur;
			} else if (lbl.equals("HOMESTRETCH")) {
				tension = 0.1;
				surprise = 0.0;
				amusement = 0.0;
				pity = 0.0;
				duration = uur;
			} else if (lbl.equals("TOPHOMESTRETCH")) {
				tension = 0.1;
				surprise = 0.0;
				amusement = 0.0;
				pity = 0.0;
				duration = uur;
			} else if (lbl.equals("CLUBHOUSETURN")) {
				tension = 0.05;
				surprise = 0.0;
				amusement = 0.0;
				pity = 0.0;
				duration = uur;
			} else if (lbl.equals("FIRSTHALF")) {
				tension = 0.05;
				surprise = 0.0;
				amusement = 0.0;
				pity = 0.0;
				duration = uur;
			} else if (lbl.equals("BACKSTRETCH")) {
				tension = 0.05;
				surprise = 0.0;
				amusement = 0.0;
				pity = 0.0;
				duration = uur;
			} else if (lbl.equals("TOPBACKSTRETCH")) {
				tension = 0.05;
				surprise = 0.0;
				amusement = 0.0;
				pity = 0.0;
				duration = uur;
			} else if (lbl.equals("FARTURN")) {
				tension = 0.1;
				surprise = 0.0;
				amusement = 0.0;
				pity = 0.0;
				duration = uur;
			}
			// important
			else if (lbl.equals("QUARTERPOLE")) {
				tension = 0.15;
				surprise = 0.0;
				amusement = 0.0;
				pity = 0.0;
				duration = uur;
			}
			// important
			else if (lbl.equals("FINALSTRETCH")) {
				tension = 0.1;
				surprise = 0.0;
				amusement = 0.0;
				pity = 0.0;
				duration = uur;
			}
			// important
			else if (lbl.equals("EIGHTHPOLE")) {
				tension = 0.1;
				surprise = 0.0;
				amusement = 0.0;
				pity = 0.0;
				duration = uur;
			} else if (lbl.equals("TOPFINALSTRETCH")) {
				tension = 0.0;
				surprise = 0.0;
				amusement = 0.0;
				pity = 0.0;
				duration = uur;
			} else if (lbl.equals("FINAL100")) {
				tension = 0.0;
				surprise = 0.0;
				amusement = 0.0;
				pity = 0.0;
				duration = uur;
			} else if (lbl.equals("FINISHLINE")) {
				tension = 0.0;
				surprise = 0.0;
				amusement = 0.0;
				pity = 0.0;
				duration = uur;
			}
		}

		// the passed time is in general also very important for the tension
		// level of the race
		else if (type.equals("TIME")) {
			if (lbl.equals("QUARTER")) {
				tension = 0.1;
				surprise = 0.0;
				amusement = 0.0;
				pity = 0.0;
				duration = uur;
			} else if (lbl.equals("HALF")) {
				tension = 0.1;
				surprise = 0.0;
				amusement = 0.0;
				pity = 0.0;
				duration = uur;
			} else if (lbl.equals("FINISHPOLE")) {
				tension = 0.1;
				surprise = 0.0;
				amusement = 0.0;
				pity = 0.0;
				duration = uur;
			}
		}

		else if (type.equals("SPEED")) {
			if (lbl.equals("UP")) {
				tension = 0.1;
				surprise = 0.2;
				amusement = 0.1;
				pity = 0.01;
				duration = 4000;
			} else if (lbl.equals("DOWN")) {
				tension = 0.05;
				surprise = 0.1;
				amusement = 0.05;
				pity = 0.1;
				duration = 4000;
			} else if (lbl.equals("STOP")) {
				tension = 0.7;
				surprise = 0.6;
				amusement = 0.3;
				pity = 0.9;
				duration = 6000;
			}
		}

		// gap events are often clustered together with an overtake event in the
		// middle
		else if (type.equals("GAP")) {
			if (lbl.equals("SMALL")) {
				tension = 0.04;
				surprise = 0.01;
				amusement = 0.05;
				pity = 0.01;
				duration = 3000;
			} else if (lbl.equals("LARGE")) {
				tension = 0.02;
				surprise = 0.01;
				amusement = 0.01;
				pity = 0.01;
				duration = 3000;
			}
		}

		else if (type.equals("OVERTAKE")) {
			tension = 0.15;
			surprise = 0.1;
			amusement = 0.3;
			pity = 0.1;
			duration = 4000;
		}

		// ranking events come in a burst, this gives a potential problem when
		// adding emotions of several events
		else if (type.equals("RANK")) {
			if (lbl.equals("1")) {
				tension = 0.05;
				surprise = 0.1;
				amusement = 0.1;
				pity = 0.01;
				duration = 3000;
			} else if (lbl.equals("2")) {
				tension = 0.03;
				surprise = 0.01;
				amusement = 0.01;
				pity = 0.01;
				duration = 3000;
			} else if (lbl.equals("3")) {
				tension = 0.02;
				surprise = 0.01;
				amusement = 0.01;
				pity = 0.01;
				duration = 3000;
			} else if (lbl.equals("4")) {
				tension = 0.01;
				surprise = 0.01;
				amusement = 0.01;
				pity = 0.1;
				duration = 3000;
			}
		}

		else if (type.equals("PRESTART")) {
			tension = 0.2;
			surprise = 0.01;
			amusement = 0.1;
			pity = 0.01;
			duration = 10000;
		}

		else if (type.equals("START")) {
			tension = 0.3;
			surprise = 0.1;
			amusement = 0.3;
			pity = 0.01;
			duration = 4000;
		}

		// every horse gets a finish event
		else if (type.equals("FINISH")) {
			if (lbl.equals("1")) {
				tension = 0.3;
				surprise = 0.3;
				amusement = 0.3;
				pity = 0.01;
				duration = 2000;
			} else if (lbl.equals("2")) {
				tension = 0.1;
				surprise = 0.1;
				amusement = 0.1;
				pity = 0.01;
				duration = 2000;
			} else if (lbl.equals("3")) {
				tension = 0.05;
				surprise = 0.01;
				amusement = 0.01;
				pity = 0.1;
				duration = 2000;
			} else if (lbl.equals("4")) {
				tension = 0.05;
				surprise = 0.01;
				amusement = 0.01;
				pity = 0.2;
				duration = 3000;
			}
		}

		else if (type.equals("POSTFINISH")) {
			tension = -0.3;
			duration = uur;
		}

		// of het nou type "TERMINOLOGY", type "BACKGROUND" of alleen label
		// "BACKGROUND" is,
		// de background events zouden door onderstaande code moeten werken:

		else if (type.equals("TERMINOLOGY")) {
			tension = 0.05;
			surprise = 0.01;
			amusement = 0.01;
			pity = 0.01;
			duration = 3000;
		} else if (type.equals("BACKGROUND")) {
			tension = 0.05;
			surprise = 0.01;
			amusement = 0.01;
			pity = 0.01;
			duration = 3000;
		} else if (lbl.equals("BACKGROUND")) {
			tension = 0.05;
			surprise = 0.01;
			amusement = 0.01;
			pity = 0.01;
			duration = 3000;
		}

		// error-check
		if (duration == 1) {
			log.warning("Following RaceEvent not recognized by MMM: "
					+ raceevent.toString() + " -- default parameters selected");
			tension = 0.1;
			surprise = 0.2;
			amusement = 0.1;
			pity = 0.01;
			duration = 4000;
		}

		generatedEmotions.put("tension", tension);
		generatedEmotions.put("surprise", surprise);
		generatedEmotions.put("amusement", amusement);
		generatedEmotions.put("pity", pity);
		this.duration = duration;

		applyImportance();
	}

}
