/* 
 * Copyright (C) 2009 HMI Group, University of Twente.
 *
 * @author Franois L.A. Knoppel <[alias] francois [on server at] knoppel.nl>
 * @author Almer S. Tigelaar <[alias] almer [on server at] gnome.org>
 * @author Danny Oude Bos <[alias] d.oudebos [on server at] utwente.nl>
 * @author Thijs Alofs <[alias] t.alofs [on server at] alumnus.utwente.nl>
 *
 * This file is part of the DEIRA project, which is free software; you can redistribute it
 * and/or modify it under the terms of the GNU "Lesser" General Public License v3 as published by the 
 * Free Software Foundation.
 */

package nl.utwente.ewi.hmi.deira.mmm;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.util.HashMap;
import java.util.StringTokenizer;
import java.util.Map.Entry;
import java.util.logging.Logger;

import nl.knoppel.tools.StringTool;
import nl.utwente.ewi.hmi.deira.queue.Event;
import nl.utwente.ewi.hmi.deira.queue.VSEvent;

/**
 * A class that generates emotions for Virtual Storyteller Events
 * 
 * @author Dual Infinity
 */
public class VSEmotionalTagger extends EmotionalTagger {

	private HashMap<String, HashMap<String, Double>> phraseMap = new HashMap<String, HashMap<String, Double>>();
	private static Logger log = Logger.getLogger("deira.mmm");

	/** < Logging instance */

	public VSEmotionalTagger() {
		super();
		parsePhraseEffects();
		useImportance = true;
	}

	private void parsePhraseEffects() {
		String inputFile = "StringEmotionals.txt";

		try {

			BufferedReader inputReader = new BufferedReader(new FileReader(
					inputFile));
			String currentLine = inputReader.readLine();
			;

			while (currentLine != null) {

				StringTokenizer lineTokenizer = new StringTokenizer(
						currentLine, "|");
				if (lineTokenizer.hasMoreTokens()) {
					String phrase = lineTokenizer.nextToken().trim();
					HashMap<String, Double> phraseEffects = new HashMap<String, Double>();
					while (lineTokenizer.hasMoreTokens()) {
						String effect = lineTokenizer.nextToken().trim();
						String emotion = effect.substring(0, effect
								.indexOf('_'));
						Double amount = Double.parseDouble(effect
								.substring(effect.indexOf('_') + 1));
						phraseEffects.put(emotion, amount);
					}

					phraseMap.put(phrase, phraseEffects);
				}
				currentLine = inputReader.readLine();

			}
		} catch (IOException e) {
			log
					.severe("VSEmoTagger: Cannot read input file or other IO Exception");
		}

	}

	public void processEvent(Event event) {
		// =-- Input checking
		VSEvent vsevent;
		if (event instanceof VSEvent) {
			vsevent = (VSEvent) event;
		} else {
			return;
		}

		// =-- Variable initialization
		importanceFactor = (vsevent.getImportance() * 4.0);
		String text = vsevent.getText();

		double exclamationScore = Math.min(0.4, 0.1 * StringTool.getCount('!',
				text));

		generatedEmotions.put("tension", 0.1 + exclamationScore);
		generatedEmotions.put("surprise", 0.0);
		generatedEmotions.put("amusement", 0.0);
		generatedEmotions.put("sadness", 0.0);
		generatedEmotions.put("anger", 0.0);
		generatedEmotions.put("pity", 0.0);
		int duration = 8000;

		StringTokenizer tokenizer = new StringTokenizer(text);
		while (tokenizer.hasMoreTokens()) {
			String word = tokenizer.nextToken();
			word = StringTool.stripPunctuation(word);
			HashMap<String, Double> phraseEffects = getPhraseEffects(word);

			for (Entry<String, Double> entry : phraseEffects.entrySet()) {
				String key = entry.getKey();
				Double existingValue = generatedEmotions.get(key);
				if (existingValue != null) {
					generatedEmotions
							.put(key, existingValue + entry.getValue());
				}
			}

		}

		this.duration = duration;

		applyImportance();
	}

	private HashMap<String, Double> getPhraseEffects(String word) {
		HashMap<String, Double> phraseEffects = phraseMap.get(word);
		if (phraseEffects == null) {
			phraseEffects = new HashMap<String, Double>();
		}
		return phraseEffects;
	}

}
