/* 
 * Copyright (C) 2009 HMI Group, University of Twente.
 *
 * @author Franois L.A. Knoppel <[alias] francois [on server at] knoppel.nl>
 * @author Almer S. Tigelaar <[alias] almer [on server at] gnome.org>
 * @author Danny Oude Bos <[alias] d.oudebos [on server at] utwente.nl>
 * @author Thijs Alofs <[alias] t.alofs [on server at] alumnus.utwente.nl>
 *
 * This file is part of the DEIRA project, which is free software; you can redistribute it
 * and/or modify it under the terms of the GNU "Lesser" General Public License v3 as published by the 
 * Free Software Foundation.
 */

package nl.utwente.ewi.hmi.deira.mmm;

import java.util.HashMap;
import java.util.Map.Entry;
import java.util.logging.Logger;

import nl.utwente.ewi.hmi.deira.queue.Event;
import nl.utwente.ewi.hmi.deira.queue.VVCEvent;

/**
 * A class that generates emotions for Horse Race Events
 * 
 * @author Dual Infinity
 */
public class VVCEmotionalTagger extends EmotionalTagger {

	private static Logger log = Logger.getLogger("deira.vvcet");

	/** < Logging instance */

	public VVCEmotionalTagger() {
		super();
		useImportance = true;
	}

	public void processEvent(Event event) {
		// =-- Input checking
		VVCEvent vvcevent;
		if (event instanceof VVCEvent) {
			vvcevent = (VVCEvent) event;
		} else {
			return;
		}

		// =-- Variable initialization
		String type = vvcevent.getType();
		float multFactor = 0.5f;
		int duration = 4000;
		double tension = importanceFactor;
		double amusement = 0.001;

		if (type.equals("goal")) {
			tension = 1.0;
			amusement = 0.8;
		} else if (type.equals("freeball")) {
			tension = 0.5;
			amusement = 0.3;
		} else if (type.equals("penaltykick")) {
			tension = 0.8;
		} else if (type.equals("goalkick")) {
			tension = 0.3;
		} else if (type.equals("freekick")) {
			tension = 0.6;
		} else if (type.equals("kick-off")) {
			tension = 0.1;
		} else if (type.equals("boring")) {
			tension = 0.0;
		} else if (type.equals("start")) {
			tension = 0.3;
		} else if (type.equals("end")) {
			tension = 0.0;
			amusement = 0.6;
		} else if (type.equals("enddraw")) {
			tension = 0.0;
			amusement = 0.0;
		} else if (type.equals("matchkick-off")) {
			tension = 0.5;
			amusement = 0.3;
		} else if (type.equals("goaldisallowed")) {
			tension = 6.0;
			amusement = 0.2;
		} else if (type.equals("freeballtaken")) {
			tension = 0.3;
			amusement = 0.3;
		} else if (type.equals("penaltykicktaken")) {
			tension = 0.3;
			amusement = 0.3;
		} else if (type.equals("penaltykickmiss")) {
			tension = 0.3;
			amusement = 0.3;
		} else if (type.equals("worthlesspenaltykick")) {
			tension = 0.0;
			amusement = 0.0;
		} else if (type.equals("keepersave")) {
			tension = 1.0;
			amusement = 0.8;
		} else if (type.equals("shotontarget")) {
			tension = 0.8;
			amusement = 0.6;
		} else if (type.equals("goalkicktaken")) {
			tension = 0.3;
			amusement = 0.3;
		} else if (type.equals("ballpossession")) {
			tension = 0.1;
			amusement = 0.1;
		} else if (type.equals("stats")) {
			tension = 0.2;
			amusement = 0.2;
		} else if (type.equals("owngoal")) {
			tension = 1.0;
			amusement = 0.8;
		} else if (type.equals("defend")) {
			tension = 0.7;
			amusement = 0.3;
		}

		// error-check
		if (duration == 1) {
			log.warning("Following RaceEvent not recognized by MMM: "
					+ vvcevent.toString() + " -- default parameters selected");
			tension = 0.1;
			duration = 4000;
		}

		generatedEmotions.put("tension", tension);
		generatedEmotions.put("amusement", amusement);
		this.duration = duration;

		applyFactor(multFactor);

	}

	public void applyFactor(float multFactor) {
		// =- Initialize storage map
		HashMap<String, Double> updatedemotions = new HashMap<String, Double>();

		// =-- Iterate previously generated emotions map
		for (Entry<String, Double> currentemotion : generatedEmotions
				.entrySet()) {
			updatedemotions.put(currentemotion.getKey(), currentemotion
					.getValue()
					* multFactor);
		}

		// =-- Update previously generated emotions
		generatedEmotions = updatedemotions;
	}

}
