/* 
 * Copyright (C) 2009 HMI Group, University of Twente.
 *
 * @author Franois L.A. Knoppel <[alias] francois [on server at] knoppel.nl>
 * @author Almer S. Tigelaar <[alias] almer [on server at] gnome.org>
 * @author Danny Oude Bos <[alias] d.oudebos [on server at] utwente.nl>
 * @author Thijs Alofs <[alias] t.alofs [on server at] alumnus.utwente.nl>
 *
 * This file is part of the DEIRA project, which is free software; you can redistribute it
 * and/or modify it under the terms of the GNU "Lesser" General Public License v3 as published by the 
 * Free Software Foundation.
 */

package nl.utwente.ewi.hmi.deira.queue;

import java.util.Comparator;
import java.util.logging.Logger;

/**
 * Comparator class that compares events based on their current importance. In
 * the case that the current importance of both events is equal, the earliest
 * created event gets precedence.
 */
public class EventComparator<T> implements Comparator<T> {
	private static Logger log = Logger.getLogger("deira.queue");

	/** < Logging instance */

	/**
	 * Constructor.
	 */
	public EventComparator() {
	}

	/**
	 * Compares two RaceEvents based on their current importance.
	 * 
	 * @param o1
	 *            First RaceEvent to compare.
	 * @param o2
	 *            Second RaceEvent to compare with.
	 * @returns See Comparator interface, return value is conformant.
	 */
	public int compare(T o1, T o2) {
		try {
			float p1 = ((Event) o1).getImportance();
			float p2 = ((Event) o2).getImportance();
			// if the importance is equal, the earlier created event gets
			// precedence over the later created one
			if (p1 == p2) {
				long t1 = ((Event) o1).getCreateSystemTime();
				long t2 = ((Event) o2).getCreateSystemTime();
				return (int) (t2 - t1);
			} else
				return (int) ((p1 - p2) * 100);
			// *100 because of the truncation to int, otherwise it would not be
			// precise enough
		} catch (Exception e) {
			log
					.severe("EventComparator: Objects not of RaceEvent class. Could not be compared.");
			return 0;
		}
	}

}