/* 
 * Copyright (C) 2009 HMI Group, University of Twente.
 *
 * @author Franois L.A. Knoppel <[alias] francois [on server at] knoppel.nl>
 * @author Almer S. Tigelaar <[alias] almer [on server at] gnome.org>
 * @author Danny Oude Bos <[alias] d.oudebos [on server at] utwente.nl>
 * @author Thijs Alofs <[alias] t.alofs [on server at] alumnus.utwente.nl>
 *
 * This file is part of the DEIRA project, which is free software; you can redistribute it
 * and/or modify it under the terms of the GNU "Lesser" General Public License v3 as published by the 
 * Free Software Foundation.
 */

package nl.utwente.ewi.hmi.deira.queue;

public class EventComparatorTest {

	/**
	 * Run to test class EventComparator
	 * 
	 * @param arg
	 *            N/A
	 */
	public static void main(String[] arg) {
		EventComparator<Object> comparator = new EventComparator<Object>();
		HREvent e1;
		HREvent e2;

		// tests: e1 < e2
		// importance: small diff (0.01)
		e1 = new HREvent(1, 0, "TEST", "1", null, 4.99f, 0, null);
		e2 = new HREvent(2, 0, "TEST", "2", null, 5.0f, 0, null);
		if (comparator.compare(e1, e2) < 0)
			System.out.println("Test1a: PASS");
		else
			System.out.println("Test1a: FAILED");
		// importance: large diff
		e1 = new HREvent(1, 0, "TEST", "1", null, 0.0f, 0, null);
		e2 = new HREvent(2, 0, "TEST", "2", null, 5.0f, 0, null);
		if (comparator.compare(e1, e2) < 0)
			System.out.println("Test1b: PASS");
		else
			System.out.println("Test1b: FAILED");
		// importance: equal, e1 created after e2
		e2 = new HREvent(2, 0, "TEST", "2", null, 5.0f, 0, null);
		synchronized (e2) { // 'uitspreektijd'
			try {
				e2.wait(20);
			} catch (InterruptedException ex) {
			}
		}
		e1 = new HREvent(1, 0, "TEST", "1", null, 5.0f, 0, null);
		if (comparator.compare(e1, e2) < 0)
			System.out.println("Test1c: PASS");
		else
			System.out.println("Test1c: FAILED: " + e1.getCreateTime() + " "
					+ e2.getCreateTime());

		// tests: e1 > e2
		// importance: small diff (0.01)
		e1 = new HREvent(1, 0, "TEST", "1", null, 5.0f, 0, null);
		e2 = new HREvent(2, 0, "TEST", "2", null, 4.99f, 0, null);
		if (comparator.compare(e1, e2) > 0)
			System.out.println("Test2a: PASS");
		else
			System.out.println("Test2a: FAILED");
		// importance: large diff
		e1 = new HREvent(1, 0, "TEST", "1", null, 5.0f, 0, null);
		e2 = new HREvent(2, 0, "TEST", "2", null, 0.0f, 0, null);
		if (comparator.compare(e1, e2) > 0)
			System.out.println("Test2b: PASS");
		else
			System.out.println("Test2b: FAILED");
		// importance: equal, e2 created after e1
		e1 = new HREvent(1, 0, "TEST", "1", null, 5.0f, 0, null);
		synchronized (e1) { // 'uitspreektijd'
			try {
				e1.wait(20);
			} catch (InterruptedException ex) {
			}
		}
		e2 = new HREvent(2, 0, "TEST", "2", null, 5.0f, 0, null);
		if (comparator.compare(e1, e2) > 0)
			System.out.println("Test2c: PASS");
		else
			System.out.println("Test2c: FAILED: " + e1.getCreateTime() + " "
					+ e2.getCreateTime());

		// tests: e1 == e2, qua importance & creation time
		e2 = new HREvent(2, 0, "TEST", "2", null, 5.0f, 0, null);
		e1 = new HREvent(1, 0, "TEST", "1", null, 5.0f, 0, null);
		if (comparator.compare(e1, e2) == 0)
			System.out.println("Test3: PASS");
		else
			System.out.println("Test3: FAILED");
	}
}
