/* 
 * Copyright (C) 2009 HMI Group, University of Twente.
 *
 * @author Franois L.A. Knoppel <[alias] francois [on server at] knoppel.nl>
 * @author Almer S. Tigelaar <[alias] almer [on server at] gnome.org>
 * @author Danny Oude Bos <[alias] d.oudebos [on server at] utwente.nl>
 * @author Thijs Alofs <[alias] t.alofs [on server at] alumnus.utwente.nl>
 *
 * This file is part of the DEIRA project, which is free software; you can redistribute it
 * and/or modify it under the terms of the GNU "Lesser" General Public License v3 as published by the 
 * Free Software Foundation.
 */

package nl.utwente.ewi.hmi.deira.queue;

import java.util.HashMap;

/**
 * An event that occurred in the race, as recognized by the RaceAnalysis module.
 * Possibly tagged with additional information by the other modules (MMM, TGM,
 * SAM, FAM).
 * 
 * version 2: added general tag system so any module can add and request tags.
 */
public class VVCEvent extends Event {
	private HashMap<String, String> VVCparams;

	/**
	 * Constructor.
	 * 
	 * @param id
	 *            Event identifier.
	 * @param time
	 *            Event creation time counting from start of the race, in
	 *            milliseconds.
	 * @param type
	 *            Event type.
	 * @param label
	 *            Event label (or subtype).
	 * @param importance
	 *            Basic event importance.
	 * @param decay
	 *            Importance decay per second.
	 * @param VVCParameters
	 *            Prepared event parameters.
	 */
	public VVCEvent(int id, long time, String type, String label,
			float importance, float decay, HashMap<String, String> VVCparams) {
		super(id, time, type, label, importance, decay);
		this.VVCparams = VVCparams;
	}

	/**
	 * Tests whether two events are similar. Two events are similar if the type,
	 * the label and the actors are the same.
	 * 
	 * @param e
	 *            The race event to compare this event with.
	 * @return True if e is similar to this race event.
	 */
	public boolean isSimilar(Event e) {
		if (!(e instanceof VVCEvent))
			return false;
		VVCEvent vvce = (VVCEvent) e;

		if (!(getType().equals(vvce.getType())))
			return false;
		if (!(getLabel().equals(vvce.getLabel())))
			return false;
		if (!(getVVCparams().equals(vvce.getVVCparams())))
			return false;

		return true;
	}

	/**
	 * Returns the VVC specific parameters
	 * 
	 * @return
	 */
	public HashMap<String, String> getVVCparams() {
		return VVCparams;
	}

}
