/* 
 * Copyright (C) 2009 HMI Group, University of Twente.
 *
 * @author Franois L.A. Knoppel <[alias] francois [on server at] knoppel.nl>
 * @author Almer S. Tigelaar <[alias] almer [on server at] gnome.org>
 * @author Danny Oude Bos <[alias] d.oudebos [on server at] utwente.nl>
 * @author Thijs Alofs <[alias] t.alofs [on server at] alumnus.utwente.nl>
 *
 * This file is part of the DEIRA project, which is free software; you can redistribute it
 * and/or modify it under the terms of the GNU "Lesser" General Public License v3 as published by the 
 * Free Software Foundation.
 */

package nl.utwente.ewi.hmi.deira.tgm;

import java.util.ArrayList;

/**
 * Container class for TextFragments.
 */

public class TextFragment {
	/** The words this TextFragment is made out of */
	private ArrayList<String> text = new ArrayList<String>();

	/** The tension bounds for which this Text Fragment is valid */
	private ArrayList<EmotionalBoundPair> emotionalBounds = new ArrayList<EmotionalBoundPair>();

	public TextFragment(ArrayList<EmotionalBoundPair> emotionalBounds,
			ArrayList<String> text) {
		super();
		this.setEmotionalBounds(emotionalBounds);
		this.text = text;
	}

	public TextFragment(ArrayList<EmotionalBoundPair> emotionalBounds,
			String text) {
		super();
		this.setEmotionalBounds(emotionalBounds);
		this.text.add(text);
	}

	public TextFragment() {
	}

	/**
	 * Method that overrides Object.equals(). Returns true if all attributes of
	 * both objects are equal.
	 */
	public boolean equals(Object other) {
		// Other typed objects are unequal
		if (!(other instanceof TextFragment)) {
			return false;
		} else {
			TextFragment S_other = (TextFragment) other;

			// All attributes match
			boolean match = true;

			ArrayList<String> other_text = S_other.getText();
			for (int i = 0; i < text.size(); i++) {
				if (other_text.get(i) != text.get(i)) {
					return false;
				}
			}

			ArrayList<EmotionalBoundPair> other_emos = S_other
					.getEmotionalBounds();
			for (int i = 0; i < emotionalBounds.size(); i++) {
				if (!other_emos.contains(emotionalBounds.get(i))) {
					return false;
				}
			}

			return match;

		}

	}

	/**
	 * Method to create a deep copy of this TextFragment.
	 * 
	 * @return A new TextFragment-object with the same attributes as this
	 *         TextFragment.
	 */
	public TextFragment copy() {
		ArrayList<String> newText = new ArrayList<String>();
		ArrayList<EmotionalBoundPair> newBounds = new ArrayList<EmotionalBoundPair>();

		for (String currenttext : text) {
			newText.add(currenttext);
		}

		for (EmotionalBoundPair currentbounds : emotionalBounds) {
			newBounds.add(currentbounds);
		}

		TextFragment newfragment = new TextFragment(newBounds, newText);
		return newfragment;
	}

	/**
	 * @return Returns the text.
	 */
	public ArrayList<String> getText() {
		return text;
	}

	/**
	 * @param text
	 *            The text to set.
	 */
	public void setText(ArrayList<String> text) {
		this.text = text;
	}

	public void addWords(ArrayList<String> words) {
		for (String word : words) {
			this.addWord(word);
		}
	}

	public void addWord(String word) {
		this.text.add(word);
	}

	/**
	 * Method to get a string representation of this TextFragment.
	 */
	public String toString() {
		String string = "";
		for (String currentword : this.text) {
			string += "[" + currentword + "]";
		}

		return string;
	}

	public void setEmotionalBounds(ArrayList<EmotionalBoundPair> emotionalBounds) {
		this.emotionalBounds = emotionalBounds;
	}

	public ArrayList<EmotionalBoundPair> getEmotionalBounds() {
		return emotionalBounds;
	}

	public void addEmotionalBound(EmotionalBoundPair newemotionalbound) {
		emotionalBounds.add(newemotionalbound);
	}
}