/* 
 * Copyright (C) 2009 HMI Group, University of Twente.
 *
 * @author Franois L.A. Knoppel <[alias] francois [on server at] knoppel.nl>
 * @author Almer S. Tigelaar <[alias] almer [on server at] gnome.org>
 * @author Danny Oude Bos <[alias] d.oudebos [on server at] utwente.nl>
 * @author Thijs Alofs <[alias] t.alofs [on server at] alumnus.utwente.nl>
 *
 * This file is part of the DEIRA project, which is free software; you can redistribute it
 * and/or modify it under the terms of the GNU "Lesser" General Public License v3 as published by the 
 * Free Software Foundation.
 */

package nl.utwente.ewi.hmi.deira.tgm;

import java.util.ArrayList;
import java.util.HashMap;

import natlang.rdg.surfacerealizer.SurfaceRealizer;
import nl.utwente.ewi.hmi.deira.db.PersonalityDB;
import nl.utwente.ewi.hmi.deira.mmm.MMM;
import nl.utwente.ewi.hmi.deira.queue.Event;
import nl.utwente.ewi.hmi.deira.queue.EventQueue;
import nl.utwente.ewi.hmi.deira.queue.VSEvent;

public class VSTextGenerator extends TextGenerator {

	public VSTextGenerator(PersonalityDB pdb, MMM mmm, EventQueue queue) {
		super(pdb, mmm, queue);

	}

	/**
	 * Converts the data that is accessible within the event to parameters that
	 * are usable in the text generation process. The actual conversion process
	 * is done in iam.vvciam.Eventgenerator.transformVSEvent()
	 * 
	 * @param current
	 * @return a map of parameter => value
	 */
	protected HashMap<String, String> prepareParameters(Event p_event) {
		// =-- Checken of we met een VSEvent te maken hebben
		if (!(p_event instanceof VSEvent))
			return null;
		VSEvent event = (VSEvent) p_event;

		// =-- Outputvariabele prepareren
		HashMap<String, String> parameters = event.getVSparams();

		// =-- Parameters teruggeven
		return parameters;
	}

	/**
	 * Processes an event to sentences
	 * 
	 * @param event
	 * @return the produced sentences
	 */
	protected ArrayList<TextFragment> processEvent(Event p_event) {
		// =-- Checken of we met een VSEvent te maken hebben
		if (!(p_event instanceof VSEvent))
			return null;
		VSEvent event = (VSEvent) p_event;

		// =-- Extract graph to process
		String text = event.getText();

		// =-- Outputvariabele prepareren
		ArrayList<TextFragment> textfragments = new ArrayList<TextFragment>();

		TextFragment sentence = new TextFragment();

		sentence.addWord(text);

		// =-- Add the textfragment to the return variable
		textfragments.add(sentence);

		// =-- TextFragments teruggeven
		return textfragments;

	}

	public String getModuleName() {
		return "VSTGM";
	}

}