/* 
 * Copyright (C) 2009 HMI Group, University of Twente.
 *
 * @author Franois L.A. Knoppel <[alias] francois [on server at] knoppel.nl>
 * @author Almer S. Tigelaar <[alias] almer [on server at] gnome.org>
 * @author Danny Oude Bos <[alias] d.oudebos [on server at] utwente.nl>
 * @author Thijs Alofs <[alias] t.alofs [on server at] alumnus.utwente.nl>
 *
 * This file is part of the DEIRA project, which is free software; you can redistribute it
 * and/or modify it under the terms of the GNU "Lesser" General Public License v3 as published by the 
 * Free Software Foundation.
 */

package nl.utwente.ewi.hmi.deira.tgm;

import java.util.ArrayList;
import java.util.HashMap;

import nl.utwente.ewi.hmi.deira.db.PersonalityDB;
import nl.utwente.ewi.hmi.deira.mmm.MMM;
import nl.utwente.ewi.hmi.deira.queue.Event;
import nl.utwente.ewi.hmi.deira.queue.EventQueue;
import nl.utwente.ewi.hmi.deira.queue.VVCEvent;

public class VVCTextGenerator extends TextGenerator {

	public VVCTextGenerator(PersonalityDB pdb, MMM mmm, EventQueue queue) {
		super(pdb, mmm, queue);

		this.setGrammar(null);
	}

	/**
	 * Converts the data that is accessible within the event to parameters that
	 * are usable in the text generation process. The actual conversion process
	 * is done in iam.vvciam.Eventgenerator.transformVSEvent()
	 * 
	 * @param current
	 * @return a map of parameter => value
	 */
	protected HashMap<String, String> prepareParameters(Event p_event) {
		// =-- Checken of we met een VVCEvent te maken hebben
		if (!(p_event instanceof VVCEvent))
			return null;
		VVCEvent event = (VVCEvent) p_event;

		// =-- Outputvariabele prepareren
		HashMap<String, String> parameters = event.getVVCparams();

		// =-- Parameters teruggeven
		return parameters;
	}

	/**
	 * Processes an event to sentences
	 * 
	 * @param event
	 * @return the produced sentences
	 */
	protected ArrayList<TextFragment> processEvent(Event event) {
		// =-- Outputvariabele prepareren
		ArrayList<TextFragment> textfragments = new ArrayList<TextFragment>();

		// =-- Input voor de production rules bepalen
		// Rulename (==type)
		String rulename = event.getType();

		// Parameters
		HashMap<String, String> parameters = (HashMap<String, String>) event
				.getTag("textparams");

		// =-- Input door de juiste production rules gooien.
		ArrayList<ProductionRule> applicablerules = this.grammar
				.getProductionRules(rulename);
		if (applicablerules != null) {
			for (ProductionRule currentrule : applicablerules) {
				ArrayList<TextFragment> currentfragments = currentrule
						.produce(parameters);
				if (currentfragments != null)
					textfragments.addAll(currentfragments);
			}

			// =-- TextFragments teruggeven
			return textfragments;
		} else {
			log.severe("TGM.G.pE(): no rules applicable for " + rulename);
			return null;
		}
	}

	public String getModuleName() {
		return "VVCTGM";
	}

}